/**************************************************************************//**
 * @file     DS_CM3.h
 * @brief    CMSIS Core Peripheral Access Layer Header File for
 *           DesignStart Cortex-M3 Device
 * @version  V1.0.0
 * @date     05. May 2017
 ******************************************************************************/
/* Copyright (c) 2011 - 2017 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/


#ifndef DS_CM3_H
#define DS_CM3_H

#ifdef __cplusplus
extern "C" {
#endif


/* -------------------------  Interrupt Number Definition  ------------------------ */

typedef enum IRQn
{
/* -------------------  Cortex-M3 Processor Exceptions Numbers  ------------------- */
  NonMaskableInt_IRQn           = -14,      /*  2 Non maskable Interrupt */
  HardFaULt_IRQn                = -13,      /*  3 HardFaULt Interrupt */
  MemoryManagement_IRQn         = -12,      /*  4 Memory Management Interrupt */
  BusFaULt_IRQn                 = -11,      /*  5 Bus FaULt Interrupt */
  UsageFaULt_IRQn               = -10,      /*  6 Usage FaULt Interrupt */
  SVCall_IRQn                   =  -5,      /* 11 SV Call Interrupt */
  DebugMonitor_IRQn             =  -4,      /* 12 Debug Monitor Interrupt */
  PendSV_IRQn                   =  -2,      /* 14 Pend SV Interrupt */
  SysTick_IRQn                  =  -1,      /* 15 System Tick Interrupt */

/* ----------------------  DS_CM3 Specific Interrupt Numbers  ------------------ */
  UART0_IRQn                    =  0,      /*     UART0 TX/RX combined */

  UART1_IRQn                    =  2,      /*     UART1 TX/RX combined */
  RESERVED1_IRQn                =  3,      /*     Reserved for APB slaves */
  RESERVED2_IRQn                =  4,      /*     Reserved for APB slaves */
  RTC_IRQn                      =  5,      /*     RTC */
  GPIO0_IRQn                    =  6,      /*     GPIO 0 combined */
  GPIO1_IRQn                    =  7,      /*     GPIO 1 combined */
  TIMER0_IRQn                   =  8,      /*     Timer0 */
  TIMER1_IRQn                   =  9,      /*     Timer1 */
  DUALTIMER_IRQn                = 10,      /*     Dual Timer */
  RESERVED3_IRQn                = 11,      /*     Reserved for APB slaves */
  UART_OVF_IRQn                 = 12,      /*     UART 0/1/2/3/4 overflow */
  RESERVED4_IRQn                = 13,      /*     Reserved for APB slaves */

  TSC_IRQn                      = 15,      /*     MPS2+ baseboard Touch Screen */
  GPIO0_0_IRQn                  = 16,      /*     GPIO0 individual interrupt ( 0) */
  GPIO0_1_IRQn                  = 17,      /*     GPIO0 individual interrupt ( 1) */
  GPIO0_2_IRQn                  = 18,      /*     GPIO0 individual interrupt ( 2) */
  GPIO0_3_IRQn                  = 19,      /*     GPIO0 individual interrupt ( 3) */
  GPIO0_4_IRQn                  = 20,      /*     GPIO0 individual interrupt ( 4) */
  GPIO0_5_IRQn                  = 21,      /*     GPIO0 individual interrupt ( 5) */
  GPIO0_6_IRQn                  = 22,      /*     GPIO0 individual interrupt ( 6) */
  GPIO0_7_IRQn                  = 23,      /*     GPIO0 individual interrupt ( 7) */
  GPIO0_8_IRQn                  = 24,      /*     GPIO0 individual interrupt ( 8) */
  GPIO0_9_IRQn                  = 25,      /*     GPIO0 individual interrupt ( 9) */
  GPIO0_10_IRQn                 = 26,      /*     GPIO0 individual interrupt (10) */
  GPIO0_11_IRQn                 = 27,      /*     GPIO0 individual interrupt (11) */
  GPIO0_12_IRQn                 = 28,      /*     GPIO0 individual interrupt (12) */
  GPIO0_13_IRQn                 = 29,      /*     GPIO0 individual interrupt (13) */
  GPIO0_14_IRQn                 = 30,      /*     GPIO0 individual interrupt (14) */
  GPIO0_15_IRQn                 = 31,      /*     GPIO0 individual interrupt (15) */
  RESERVED5_IRQn                = 32,      /*     Reserved for Flash */
  RESERVED6_IRQn                = 33,      /*     Reserved for Flash */
  RESERVED7_IRQn                = 34,      /*     reserved for Cordio BT4 */
  RESERVED8_IRQn                = 35,      /*     reserved for Cordio BT4 */
  RESERVED9_IRQn                = 36,      /*     reserved for Cordio BT4 */
  RESERVED10_IRQn               = 37,      /*     reserved for Cordio BT4 */
  RESERVED11_IRQn               = 38,      /*     reserved for Cordio BT4 */
  RESERVED12_IRQn               = 39,      /*     reserved for Cordio BT4 */
  RESERVED13_IRQn               = 40,      /*     reserved for Cordio BT4 */
  RESERVED14_IRQn               = 41,      /*     reserved for Cordio BT4 */
  GPIO2_IRQn                    = 42,      /*     Combined */
  GPIO3_IRQn                    = 43,      /*     Combined */
  TRNG_IRQn                     = 44,      /*     True Random Number Generator */
  UART2_IRQn                    = 45,      /*     UART 2 TX/RX compined */
  UART3_IRQn                    = 46,      /*     UART 3 TX/RX compined */
  ETHERNET_IRQn                 = 47,      /*     Ethernet Interrupt */
  I2S_IRQn                      = 48,      /*     I2S Interrupt */
  MPS2_SPI0_IRQn                = 49,      /*     SPI Header interrupt */
  MPS2_SPI1_IRQn                = 50,      /*     CLCD SPI interrupt */
  MPS2_SPI2_IRQn                = 51,      /*     ADC for Shield */
  MPS2_SPI3_IRQn                = 52,      /*     Shield0 SPI */
  MPS2_SPI4_IRQn                = 53,      /*     Shield1 SPI */
  GPIO4_IRQn                    = 54,      /*     GPIO 4 combined */
  GPIO5_IRQn                    = 55,      /*     GPIO 5 combined */
  UART4_IRQn                    = 56       /*     UART 4 TX/Rx combined */
} IRQn_Type;


/* ================================================================================ */
/* ================      Processor and Core Peripheral Section     ================ */
/* ================================================================================ */

/* -------  Start of section using anonymous unions and disabling warnings  ------- */
#if   defined (__CC_ARM)
  #pragma push
  #pragma anon_unions
#elif defined (__ICCARM__)
  #pragma language=extended
#elif defined(__ARMCC_VERSION) && (__ARMCC_VERSION >= 6010050)
  #pragma clang diagnostic push
  #pragma clang diagnostic ignored "-Wc11-extensions"
  #pragma clang diagnostic ignored "-Wreserved-id-macro"
#elif defined (__GNUC__)
  /* anonymous unions are enabled by defaULt */
#elif defined (__TMS470__)
  /* anonymous unions are enabled by defaULt */
#elif defined (__TASKING__)
  #pragma warning 586
#elif defined (__CSMC__)
  /* anonymous unions are enabled by defaULt */
#else
  #warning Not supported compiler type
#endif


/* --------  Configuration of the Cortex-M3 Processor and Core Peripherals  ------- */
#define __CM3_REV                 0x0201U   /* Core revision r2p1 */
#define __MPU_PRESENT             1         /* MPU present */
#define __NVIC_PRIO_BITS          3         /* Number of Bits used for Priority Levels */
#define __Vendor_SysTickConfig    0         /* Set to 1 if different SysTick Config is used */

#include "core_cm3.h"                       /* Processor and core peripherals */
#include "system_DS_CM3.h"                  /* System Header */


/* ================================================================================ */
/* ================       Device Specific Peripheral Section       ================ */
/* ================================================================================ */

/*------------- Universal Asynchronous Receiver Transmitter (UART) -----------*/
typedef struct /* see "Cortex-M System Design Kit Technical Reference Manual r1p0" */
{
  __IOM  uint32_t  DATA;                     /* Offset: 0x000 (R/W) Data Register    */
  __IOM  uint32_t  STATE;                    /* Offset: 0x004 (R/W) Status Register  */
  __IOM  uint32_t  CTRL;                     /* Offset: 0x008 (R/W) Control Register */
  union {
    __IM   uint32_t  INTSTATUS;              /* Offset: 0x00C (R/ ) Interrupt Status Register */
    __OM   uint32_t  INTCLEAR;               /* Offset: 0x00C ( /W) Interrupt Clear Register  */
    };
  __IOM  uint32_t  BAUDDIV;                  /* Offset: 0x010 (R/W) Baudrate Divider Register */
} DS_CM3_UART_TypeDef;

/* DS_CM3_UART DATA Register Definitions */
#define DS_CM3_UART_DATA_DATA_Pos               0                                                    /* DS_CM3_UART DATA: DATA position */
#define DS_CM3_UART_DATA_DATA_Msk              (0xFFUL /*<< DS_CM3_UART_DATA_DATA_Pos*/)             /* DS_CM3_UART DATA: DATA mask */

/* DS_CM3_UART STATE Register Definitions */
#define DS_CM3_UART_STATE_RXBO_Pos              3                                                    /* DS_CM3_UART STATE: RX buffer overrun position */
#define DS_CM3_UART_STATE_RXBO_Msk             (0x1UL << DS_CM3_UART_STATE_RXOR_Pos)                 /* DS_CM3_UART STATE: RX buffer overrun mask */

#define DS_CM3_UART_STATE_TXBO_Pos              2                                                    /* DS_CM3_UART STATE: TX buffer overrun position */
#define DS_CM3_UART_STATE_TXBO_Msk             (0x1UL << DS_CM3_UART_STATE_TXOR_Pos)                 /* DS_CM3_UART STATE: TX buffer overrun mask */

#define DS_CM3_UART_STATE_RXBF_Pos              1                                                    /* DS_CM3_UART STATE: RX buffer full position */
#define DS_CM3_UART_STATE_RXBF_Msk             (0x1UL << DS_CM3_UART_STATE_RXBF_Pos)                 /* DS_CM3_UART STATE: RX buffer full mask */

#define DS_CM3_UART_STATE_TXBF_Pos              0                                                    /* DS_CM3_UART STATE: TX buffer full position */
#define DS_CM3_UART_STATE_TXBF_Msk             (0x1UL /*<< DS_CM3_UART_STATE_TXBF_Pos*/)             /* DS_CM3_UART STATE: TX buffer full mask */

/* DS_CM3_UART CTRL Register Definitions */
#define DS_CM3_UART_CTRL_HSTM_Pos               6                                                    /* DS_CM3_UART CTRL: HSTM position */
#define DS_CM3_UART_CTRL_HSTM_Msk              (0x01UL << DS_CM3_UART_CTRL_HSTM_Pos)                 /* DS_CM3_UART CTRL: HSTM mask */

#define DS_CM3_UART_CTRL_RXORINTEN_Pos          5                                                    /* DS_CM3_UART CTRL: RX overrun interrupt enable position */
#define DS_CM3_UART_CTRL_RXORINTEN_Msk         (0x01UL << DS_CM3_UART_CTRL_RXORINTEN_Pos)            /* DS_CM3_UART CTRL: RX overrun interrupt enable mask */

#define DS_CM3_UART_CTRL_TXORINTEN_Pos          4                                                    /* DS_CM3_UART CTRL: TX overrun interrupt enable position */
#define DS_CM3_UART_CTRL_TXORINTEN_Msk         (0x01UL << DS_CM3_UART_CTRL_TXORINTEN_Pos)            /* DS_CM3_UART CTRL: TX overrun interrupt enable mask */

#define DS_CM3_UART_CTRL_RXINTEN_Pos            3                                                    /* DS_CM3_UART CTRL: RX interrupt enable position */
#define DS_CM3_UART_CTRL_RXINTEN_Msk           (0x01UL << DS_CM3_UART_CTRL_RXINTEN_Pos)              /* DS_CM3_UART CTRL: RX interrupt enable mask */

#define DS_CM3_UART_CTRL_TXINTEN_Pos            2                                                    /* DS_CM3_UART CTRL: TX interrupt enable position */
#define DS_CM3_UART_CTRL_TXINTEN_Msk           (0x01UL << DS_CM3_UART_CTRL_TXINTEN_Pos)              /* DS_CM3_UART CTRL: TX interrupt enable mask */

#define DS_CM3_UART_CTRL_RXEN_Pos               1                                                    /* DS_CM3_UART CTRL: RXEN position */
#define DS_CM3_UART_CTRL_RXEN_Msk              (0x01UL << DS_CM3_UART_CTRL_RXEN_Pos)                 /* DS_CM3_UART CTRL: RXEN mask */

#define DS_CM3_UART_CTRL_TXEN_Pos               0                                                    /* DS_CM3_UART CTRL: TXEN position */
#define DS_CM3_UART_CTRL_TXEN_Msk              (0x01UL /*<< DS_CM3_UART_CTRL_TXEN_Pos*/)             /* DS_CM3_UART CTRL: TXEN mask */

/* DS_CM3_UART INTSTATUS Register Definitions */
#define DS_CM3_UART_INTSTATUS_RXORINT_Pos       3                                                    /* DS_CM3_UART INTSTATUS: RX overrun interrupt position */
#define DS_CM3_UART_INTSTATUS_RXORINT_Msk      (0x01UL << DS_CM3_UART_INTSTATUS_RXORINT_Pos)         /* DS_CM3_UART INTSTATUS: RX overrun interrupt mask */

#define DS_CM3_UART_INTSTATUS_TXORINT_Pos       2                                                    /* DS_CM3_UART INTSTATUS: TX overrun interrupt position */
#define DS_CM3_UART_INTSTATUS_TXORINT_Msk      (0x01UL << DS_CM3_UART_INTSTATUS_TXORINT_Pos)         /* DS_CM3_UART INTSTATUS: TX overrun interrupt mask */

#define DS_CM3_UART_INTSTATUS_RXINT_Pos         1                                                    /* DS_CM3_UART INTSTATUS: RX interrupt position */
#define DS_CM3_UART_INTSTATUS_RXINT_Msk        (0x01UL << DS_CM3_UART_INTSTATUS_RXINT_Pos)           /* DS_CM3_UART INTSTATUS: RX interrupt mask */

#define DS_CM3_INTSTATUS_TXINT_Pos              0                                                    /* DS_CM3_UART INTSTATUS: TX interrupt position */
#define DS_CM3_INTSTATUS_TXINT_Msk             (0x01UL /*<< DS_CM3_UART_INTSTATUS_TXINT_Pos*/)       /* DS_CM3_UART INTSTATUS: TX interrupt mask */

/* DS_CM3_UART INTCLEAR Register Definitions */
#define DS_CM3_UART_INTCLEAR_RXORINT_Pos        3                                                    /* DS_CM3_UART INTCLEAR: RX overrun interrupt position */
#define DS_CM3_UART_INTCLEAR_RXORINT_Msk       (0x01UL << DS_CM3_UART_INTCLEAR_RXORINT_Pos)          /* DS_CM3_UART INTCLEAR: RX overrun interrupt mask */

#define DS_CM3_UART_INTCLEAR_TXORINT_Pos        2                                                    /* DS_CM3_UART INTCLEAR: TX overrun interrupt position */
#define DS_CM3_UART_INTCLEAR_TXORINT_Msk       (0x01UL << DS_CM3_UART_INTCLEAR_TXORINT_Pos)          /* DS_CM3_UART INTCLEAR: TX overrun interrupt mask */

#define DS_CM3_UART_INTCLEAR_RXINT_Pos          1                                                    /* DS_CM3_UART INTCLEAR: RX interrupt position */
#define DS_CM3_UART_INTCLEAR_RXINT_Msk         (0x01UL << DS_CM3_UART_INTCLEAR_RXINT_Pos)            /* DS_CM3_UART INTCLEAR: RX interrupt mask */

#define DS_CM3_UART_INTCLEAR_TXINT_Pos          0                                                    /* DS_CM3_UART INTCLEAR: TX interrupt position */
#define DS_CM3_UART_INTCLEAR_TXINT_Msk         (0x01UL /*<< DS_CM3_UART_INTCLEAR_TXINT_Pos*/)        /* DS_CM3_UART INTCLEAR: TX interrupt mask */

/* DS_CM3_UART BAUDDIV Register Definitions */
#define DS_CM3_UART_BAUDDIV_Pos                 0                                                    /* DS_CM3_UART BAUDDIV: BAUDDIV position */
#define DS_CM3_UART_BAUDDIV_Msk                (0xFFFFFUL /*<< DS_CM3_UART_BAUDDIV_Pos*/)            /* DS_CM3_UART BAUDDIV: BAUDDIV mask */


/*----------------------------- Timer (TIMER) -------------------------------*/
typedef struct /* see "Cortex-M System Design Kit Technical Reference Manual r1p0" */
{
  __IOM  uint32_t  CTRL;                     /* Offset: 0x000 (R/W) Control Register */
  __IOM  uint32_t  VALUE;                    /* Offset: 0x004 (R/W) Current Value Register */
  __IOM  uint32_t  RELOAD;                   /* Offset: 0x008 (R/W) Reload Value Register */
  union {
    __IM   uint32_t  INTSTATUS;              /* Offset: 0x00C (R/ ) Interrupt Status Register */
    __OM   uint32_t  INTCLEAR;               /* Offset: 0x00C ( /W) Interrupt Clear Register */
    };
} DS_CM3_TIMER_TypeDef;

/* DS_CM3_TIMER CTRL Register Definitions */
#define DS_CM3_TIMER_CTRL_INTEN_Pos             3                                                    /* DS_CM3_TIMER CTRL: interrupt enable position */
#define DS_CM3_TIMER_CTRL_INTEN_Msk            (0x01UL << DS_CM3_TIMER_CTRL_INTEN_Pos)               /* DS_CM3_TIMER CTRL: interrupt enable mask */

#define DS_CM3_TIMER_CTRL_SELEXTCLK_Pos         2                                                    /* DS_CM3_TIMER CTRL: Select external input as clock position */
#define DS_CM3_TIMER_CTRL_SELEXTCLK_Msk        (0x01UL << DS_CM3_TIMER_CTRL_SELEXTCLK_Pos)           /* DS_CM3_TIMER CTRL: Select external input as clock mask */

#define DS_CM3_TIMER_CTRL_SELEXTEN_Pos          1                                                    /* DS_CM3_TIMER CTRL: Select external input as enable position */
#define DS_CM3_TIMER_CTRL_SELEXTEN_Msk         (0x01UL << DS_CM3_TIMER_CTRL_SELEXTEN_Pos)            /* DS_CM3_TIMER CTRL: Select external input as enable mask */

#define DS_CM3_TIMER_CTRL_EN_Pos                0                                                    /* DS_CM3_TIMER CTRL: enable position */
#define DS_CM3_TIMER_CTRL_EN_Msk               (0x01UL /*<< DS_CM3_TIMER_CTRL_EN_Pos*/)              /* DS_CM3_TIMER CTRL: enable mask */

/* DS_CM3_TIMER VAL Register Definitions */
#define DS_CM3_TIMER_VAL_CURRENT_Pos            0                                                    /* DS_CM3_TIMER VALUE: current value position */
#define DS_CM3_TIMER_VAL_CURRENT_Msk           (0xFFFFFFFFUL /*<< DS_CM3_TIMER_VAL_CURRENT_Pos*/)    /* DS_CM3_TIMER VALUE: current value mask */

/* DS_CM3_TIMER RELOAD Register Definitions */
#define DS_CM3_TIMER_RELOAD_VAL_Pos             0                                                    /* DS_CM3_TIMER RELOAD: reload value position */
#define DS_CM3_TIMER_RELOAD_VAL_Msk            (0xFFFFFFFFUL /*<< DS_CM3_TIMER_RELOAD_VAL_Pos*/)     /* DS_CM3_TIMER RELOAD: reload value mask */

/* DS_CM3_TIMER INTSTATUS Register Definitions */
#define DS_CM3_TIMER_INTSTATUS_Pos              0                                                    /* DS_CM3_TIMER INTSTATUS: interrupt status position */
#define DS_CM3_TIMER_INTSTATUS_Msk             (0x01UL /*<< DS_CM3_TIMER_INTSTATUS_Pos*/)            /* DS_CM3_TIMER INTSTATUS: interrupt status mask */

/* DS_CM3_TIMER INTCLEAR Register Definitions */
#define DS_CM3_TIMER_INTCLEAR_Pos               0                                                    /* DS_CM3_TIMER INTCLEAR: interrupt clear position */
#define DS_CM3_TIMER_INTCLEAR_Msk              (0x01UL /*<< DS_CM3_TIMER_INTCLEAR_Pos*/)             /* DS_CM3_TIMER INTCLEAR: interrupt clear mask */


/*------------- Dual Timer (DUALTIMER) --------------------------------------------------*/
typedef struct
{
  __IOM uint32_t TIMER1LOAD;                 /* Offset: 0x000 (R/W) Timer 1 Load */
  __IM  uint32_t TIMER1VALUE;                /* Offset: 0x004 (R/ ) Timer 1 Counter Current Value */
  __IOM uint32_t TIMER1CONTROL;              /* Offset: 0x008 (R/W) Timer 1 Control */
  __OM  uint32_t TIMER1INTCLR;               /* Offset: 0x00C ( /W) Timer 1 Interrupt Clear */
  __IM  uint32_t TIMER1RIS;                  /* Offset: 0x010 (R/ ) Timer 1 Raw Interrupt Status */
  __IM  uint32_t TIMER1MIS;                  /* Offset: 0x014 (R/ ) Timer 1 masked Interrupt Status */
  __IOM uint32_t TIMER1BGLOAD;               /* Offset: 0x018 (R/W) Background Load Register */
        uint32_t RESERVED0;
  __IOM uint32_t TIMER2LOAD;                 /* Offset: 0x020 (R/W) Timer 2 Load */
  __IM  uint32_t TIMER2VALUE;                /* Offset: 0x024 (R/ ) Timer 2 Counter Current Value */
  __IOM uint32_t TIMER2CTRL;                 /* Offset: 0x028 (R/W) Timer 2 Control */
  __OM  uint32_t TIMER2INTCLR;               /* Offset: 0x02C ( /W) Timer 2 Interrupt Clear */
  __IM  uint32_t TIMER2RIS;                  /* Offset: 0x030 (R/ ) Timer 2 Raw Interrupt Status */
  __IM  uint32_t TIMER2MIS;                  /* Offset: 0x034 (R/ ) Timer 2 masked Interrupt Status */
  __IOM uint32_t TIMER2BGLOAD;               /* Offset: 0x038 (R/W) Background Load Register */
        uint32_t RESERVED1[945];
  __IOM uint32_t TIMERITCR;                  /* Offset: 0xF00 (R/W) Integration Test Control Register */
  __OM  uint32_t TIMERITOP;                  /* Offset: 0xF04 ( /W) Integration Test Output Set Register */
} DS_CM3_DUALTIMER_TypeDef;


typedef struct
{
  __IOM uint32_t LOAD;                       /* Offset: 0x000 (R/W) Timer Load */
  __IM  uint32_t VALUE;                      /* Offset: 0x000 (R/W) Timer Counter Current Value */
  __IOM uint32_t CTRL;                       /* Offset: 0x000 (R/W) Timer Control */
  __OM  uint32_t INTCLR;                     /* Offset: 0x000 (R/W) Timer Interrupt Clear */
  __IM  uint32_t RIS;                        /* Offset: 0x000 (R/W) Timer Raw Interrupt Status */
  __IM  uint32_t MIS;                        /* Offset: 0x000 (R/W) Timer masked Interrupt Status */
  __IOM uint32_t BGLOAD;                     /* Offset: 0x000 (R/W) Background Load Register */
} DS_CM3_DUALTIMER_SINGLE_TypeDef;

/* DS_CM3_DUALTIMER LOAD Register Definitions */
#define DS_CM3_DUALTIMER_LOAD_Pos               0                                                    /* DS_CM3_DUALTIMER LOAD: LOAD position */
#define DS_CM3_DUALTIMER_LOAD_Msk              (0xFFFFFFFFUL /*<< DS_CM3_DUALTIMER_LOAD_Pos*/)       /* DS_CM3_DUALTIMER LOAD: LOAD mask */

/* DS_CM3_DUALTIMER VALUE Register Definitions */
#define DS_CM3_DUALTIMER_VALUE_Pos              0                                                    /* DS_CM3_DUALTIMER VALUE: VALUE position */
#define DS_CM3_DUALTIMER_VALUE_Msk             (0xFFFFFFFFUL /*<< DS_CM3_DUALTIMER_VALUE_Pos*/)      /* DS_CM3_DUALTIMER VALUE: VALUE mask */

/* DS_CM3_DUALTIMER CONTROL Register Definitions */
#define DS_CM3_DUALTIMER_CONTROL_EN_Pos         7                                                    /* DS_CM3_DUALTIMER CONTROL_EN: CONTROL Enable position */
#define DS_CM3_DUALTIMER_CONTROL_EN_Msk        (0x1UL << DS_CM3_DUALTIMER_CTRL_EN_Pos)               /* DS_CM3_DUALTIMER CONTROL_EN: CONTROL Enable mask */

#define DS_CM3_DUALTIMER_CONTROL_MODE_Pos       6                                                    /* DS_CM3_DUALTIMER CONTROL_MODE: CONTROL MODE position */
#define DS_CM3_DUALTIMER_CONTROL_MODE_Msk      (0x1UL << DS_CM3_DUALTIMER_CTRL_MODE_Pos)             /* DS_CM3_DUALTIMER CONTROL_MODE: CONTROL MODE mask */

#define DS_CM3_DUALTIMER_CONTROL_INTEN_Pos      5                                                    /* DS_CM3_DUALTIMER CONTROL_INTEN: CONTROL Int Enable position */
#define DS_CM3_DUALTIMER_CONTROL_INTEN_Msk     (0x1UL << DS_CM3_DUALTIMER_CTRL_INTEN_Pos)            /* DS_CM3_DUALTIMER CONTROL_INTEN: CONTROL Int Enable mask */

#define DS_CM3_DUALTIMER_CONTROL_PRE_Pos        2                                                    /* DS_CM3_DUALTIMER CONTROL.PRE: CONTROL TimerPre position */
#define DS_CM3_DUALTIMER_CONTROL_PRE_Msk       (0x3UL << DS_CM3_DUALTIMER_CTRL_PRE_Pos)              /* DS_CM3_DUALTIMER CONTROL.PRE: CONTROL TimerPre mask */

#define DS_CM3_DUALTIMER_CONTROL_SIZE_Pos       1                                                    /* DS_CM3_DUALTIMER CONTROL.SIZE: CONTROL SIZE position */
#define DS_CM3_DUALTIMER_CONTROL_SIZE_Msk      (0x1UL << DS_CM3_DUALTIMER_CTRL_SIZE_Pos)             /* DS_CM3_DUALTIMER CONTROL.SIZE: CONTROL SIZE mask */

#define DS_CM3_DUALTIMER_CONTROL_ONESHOT_Pos    0                                                    /* DS_CM3_DUALTIMER CONTROL.ONESHOT: CONTROL ONESHOOT position */
#define DS_CM3_DUALTIMER_CONTROL_ONESHOT_Msk   (0x1UL /*<< DS_CM3_DUALTIMER_CONTROL_ONESHOT_Pos*/)   /* DS_CM3_DUALTIMER CONTROL.ONESHOT: CONTROL ONESHOOT mask */

/* DS_CM3_DUALTIMER INTCLR Register Definitions */
#define DS_CM3_DUALTIMER_INTCLR_Pos             0                                                    /* DS_CM3_DUALTIMER INTCLR: INT Clear position */
#define DS_CM3_DUALTIMER_INTCLR_Msk            (0x1UL /*<< DS_CM3_DUALTIMER_INTCLR_Pos*/)            /* DS_CM3_DUALTIMER INTCLR: INT Clear  mask */

/* DS_CM3_DUALTIMER RIS Register Definitions */
#define DS_CM3_DUALTIMER_RIS_RINT_Pos           0                                                    /* DS_CM3_DUALTIMER RIS.RINT: Raw Int Status position */
#define DS_CM3_DUALTIMER_RIS_RINT_Msk          (0x1UL /*<< DS_CM3_DUALTIMER_RIS_RINT_Pos*/)          /* DS_CM3_DUALTIMER RIS.RINT: Raw Int Status mask */

/* DS_CM3_DUALTIMER MIS Register Definitions */
#define DS_CM3_DUALTIMER_MIS_INT_Pos            0                                                    /* DS_CM3_DUALTIMER MIS.INT: mask Int Status position */
#define DS_CM3_DUALTIMER_MIS_INT_Msk           (0x1UL /*<< DS_CM3_DUALTIMER_MIS_INT_Pos*/)           /* DS_CM3_DUALTIMER MIS.INT: mask Int Status mask */

/* DS_CM3_DUALTIMER BGLOAD Register Definitions */
#define DS_CM3_DUALTIMER_BGLOAD_Pos             0                                                    /* DS_CM3_DUALTIMER BGLOAD: Background Load position */
#define DS_CM3_DUALTIMER_BGLOAD_Msk            (0xFFFFFFFFUL /*<< DS_CM3_DUALTIMER_BGLOAD_Pos*/)     /* DS_CM3_DUALTIMER BGLOAD: Background Load mask */

/* DS_CM3_DUALTIMER TIMERITCR Register Definitions */
#define DS_CM3_DUALTIMER_TIMERITCR_ITMEN_Pos    0                                                    /* DS_CM3_DUALTIMER TIMERITCR.ITMEN: Integration Test Mode Enable position */
#define DS_CM3_DUALTIMER_TIMERITCR_ITMEN_Msk   (0x1UL /*<< DS_CM3_DUALTIMER_TIMERITCR_ITMEN_Pos*/)   /* DS_CM3_DUALTIMER TIMERITCR.ITMEN: Integration Test Mode Enable mask */

/* DS_CM3_DUALTIMER TIMERITOP Register Definitions */
#define DS_CM3_DUALTIMER_TIMERITOP_TIMINT2_Pos  1                                                    /* DS_CM3_DUALTIMER TIMERITOP.TIMINT2: Integration Test TIMINT2 value position */
#define DS_CM3_DUALTIMER_TIMERITOP_TIMINT2_Msk (0x1UL << DS_CM3_DUALTIMER_TIMERITOP_TIMINT2_Pos)     /* DS_CM3_DUALTIMER TIMERITOP.TIMINT2: Integration Test TIMINT2 value mask */

#define DS_CM3_DUALTIMER_TIMERITOP_TIMINT1_Pos  0                                                    /* DS_CM3_DUALTIMER TIMERITOP.TIMINT1: Integration Test TIMINT1 value position */
#define DS_CM3_DUALTIMER_TIMERITOP_TIMINT1_Msk (0x1UL /*<< DS_CM3_DUALTIMER_TIMERITOP_TIMINT1_Pos*/) /* DS_CM3_DUALTIMER TIMERITOP.TIMINT1: Integration Test TIMINT1 value mask */



/*------------------- Watchdog ----------------------------------------------*/
typedef struct /* see "Cortex-M System Design Kit Technical Reference Manual r1p0" */
{
  __IOM  uint32_t  LOAD;                    /* Offset: 0x000 (R/W) Load Register */
  __IOM  uint32_t  VALUE;                   /* Offset: 0x004 (R/ ) Value Register */
  __IOM  uint32_t  CONTROL;                 /* Offset: 0x008 (R/W) Control Register */
  __IOM  uint32_t  INTCLR;                  /* Offset: 0x00C ( /W) Clear Interrupt Register */
  __IOM  uint32_t  RIS;                     /* Offset: 0x010 (R/ ) Raw Interrupt Status Register */
  __IOM  uint32_t  MIS;                     /* Offset: 0x014 (R/ ) Interrupt Status Register */
         uint32_t  RESERVED0[762];
  __IOM  uint32_t  LOCK;                    /* Offset: 0xC00 (R/W) Lock Register */
         uint32_t  RESERVED1[191];
  __IOM  uint32_t  ITCR;                    /* Offset: 0xF00 (R/W) Integration Test Control Register */
  __IOM  uint32_t  ITOP;                    /* Offset: 0xF04 ( /W) Integration Test Output Set Register */
} DS_CM3_WATCHDOG_TypeDef;

/* DS_CM3_WATCHDOG LOAD Register Definitions */
#define DS_CM3_WATCHDOG_LOAD_Pos                0                                                    /* DS_CM3_WATCHDOG LOAD: LOAD position */
#define DS_CM3_WATCHDOG_LOAD_Msk               (0xFFFFFFFFUL /*<< DS_CM3_WATCHDOG_LOAD_Pos*/)        /* DS_CM3_WATCHDOG LOAD: LOAD mask */

/* DS_CM3_WATCHDOG VALUE Register Definitions */
#define DS_CM3_WATCHDOG_VALUE_Pos               0                                                    /* DS_CM3_WATCHDOG VALUE: VALUE position */
#define DS_CM3_WATCHDOG_VALUE_Msk              (0xFFFFFFFFUL /*<< DS_CM3_WATCHDOG_VALUE_Pos*/)       /* DS_CM3_WATCHDOG VALUE: VALUE mask */

/* DS_CM3_WATCHDOG CONTROL Register Definitions */
#define DS_CM3_WATCHDOG_CONTROL_RESEN_Pos       1                                                    /* DS_CM3_WATCHDOG CONTROL.RESEN: Reset Output Enable position */
#define DS_CM3_WATCHDOG_CONTROL_RESEN_Msk      (0x1UL << DS_CM3_WATCHDOG_CONTROL_RESEN_Pos)          /* DS_CM3_WATCHDOG CONTROL.RESEN: Reset Output Enable mask */

#define DS_CM3_WATCHDOG_CONTROL_INTEN_Pos       0                                                    /* DS_CM3_WATCHDOG CONTROL.INTEN: Int Enable position */
#define DS_CM3_WATCHDOG_CONTROL_INTEN_Msk      (0x1UL /*<< DS_CM3_WATCHDOG_CONTROL_INTEN_Pos*/)      /* DS_CM3_WATCHDOG CONTROL.INTEN: Int Enable mask */

/* DS_CM3_WATCHDOG INTCLR Register Definitions */
#define DS_CM3_WATCHDOG_INTCLR_Pos              0                                                    /* DS_CM3_WATCHDOG INTCLR: Interrutp clear position */
#define DS_CM3_WATCHDOG_INTCLR_Msk             (0x1UL /*<< DS_CM3_WATCHDOG_INTCLR_Pos*/)             /* DS_CM3_WATCHDOG INTCLR: Interrutp clear mask */

/* DS_CM3_WATCHDOG RIS Register Definitions */
#define DS_CM3_WATCHDOG_RIS_RINT_Pos            0                                                    /* DS_CM3_WATCHDOG RIS.RINT: Raw Watchdog Interrupt position */
#define DS_CM3_WATCHDOG_RIS_RINT_Msk           (0x1UL /*<< DS_CM3_WATCHDOG_RIS_RINT_Pos*/)           /* DS_CM3_WATCHDOG RIS.RINT: Raw Watchdog Interrupt mask */

/* DS_CM3_WATCHDOG MIS Register Definitions */
#define DS_CM3_WATCHDOG_MIS_INT_Pos             0                                                    /* DS_CM3_WATCHDOG MIS.INT: Watchdog Interrupt position */
#define DS_CM3_WATCHDOG_MIS_INT_Msk             (0x1UL /*<< DS_CM3_WATCHDOG_MIS_INT_Pos*/)           /* DS_CM3_WATCHDOG MIS.INT: Watchdog Interrupt mask */

/* DS_CM3_WATCHDOG LOCK Register Definitions */
#define DS_CM3_WATCHDOG_LOCK_WEN_Pos            0                                                    /* DS_CM3_WATCHDOG LOCK.WEN: Register write enable status position */
#define DS_CM3_WATCHDOG_LOCK_WEN_Msk           (0x1UL /*<< DS_CM3_WATCHDOG_LOCK_WEN_Pos*/)           /* DS_CM3_WATCHDOG LOCK.WEN: Register write enable status mask */

/* DS_CM3_WATCHDOG INTEGTESTEN Register Definitions */
#define DS_CM3_WATCHDOG_ITCR_ITMEN_Pos          0                                                    /* DS_CM3_WATCHDOG ITCR.ITMEN: Integration Test Mode Enable position */
#define DS_CM3_WATCHDOG_ITCR_ITMEN_Msk         (0x1UL /*<< DS_CM3_WATCHDOG_ITCR_ITMEN_Pos*/)         /* DS_CM3_WATCHDOG ITCR.ITMEN: Integration Test Mode Enable mask */

/* DS_CM3_WATCHDOG INTEGTESTOUTSET Register Definitions */
#define DS_CM3_WATCHDOG_ITOP_WDOGINT_Pos        1                                                    /* DS_CM3_WATCHDOG ITOP.WDOGINT: Integration Test WDOGINT value position */
#define DS_CM3_WATCHDOG_ITOP_WDOGINT_Msk       (0x1UL << DS_CM3_WATCHDOG_ITOP_WDOGINT_Pos)           /* DS_CM3_WATCHDOG ITOP.WDOGINT: Integration Test WDOGINT value mask */

#define DS_CM3_WATCHDOG_ITOP_WDOGRES_Pos        0                                                    /* DS_CM3_WATCHDOG ITOP.WDOGRES: Integration Test WDOGRES value position */
#define DS_CM3_WATCHDOG_ITOP_WDOGRES_Msk       (0x1UL /*<< DS_CM3_WATCHDOG_ITOP_WDOGRES_Pos*/)       /* DS_CM3_WATCHDOG ITOP.WDOGRES: Integration Test WDOGRES value mask */


/*-------------------- General Purpose Input Output (GPIO) -------------------*/
typedef struct /* see "Cortex-M System Design Kit Technical Reference Manual r1p0" */
{
  __IOM  uint32_t  DATA;                     /* Offset: 0x000 (R/W) DATA Register */
  __IOM  uint32_t  DATAOUT;                  /* Offset: 0x004 (R/W) Data Output Latch Register */
         uint32_t  RESERVED0[2];
  __IOM  uint32_t  OUTENSET;                 /* Offset: 0x010 (R/W) Output Enable Set Register */
  __IOM  uint32_t  OUTENCLR;                 /* Offset: 0x014 (R/W) Output Enable Clear Register */
  __IOM  uint32_t  ALTFUNCSET;               /* Offset: 0x018 (R/W) Alternate Function Set Register */
  __IOM  uint32_t  ALTFUNCCLR;               /* Offset: 0x01C (R/W) Alternate Function Clear Register */
  __IOM  uint32_t  INTENSET;                 /* Offset: 0x020 (R/W) Interrupt Enable Set Register */
  __IOM  uint32_t  INTENCLR;                 /* Offset: 0x024 (R/W) Interrupt Enable Clear Register */
  __IOM  uint32_t  INTTYPESET;               /* Offset: 0x028 (R/W) Interrupt Type Set Register */
  __IOM  uint32_t  INTTYPECLR;               /* Offset: 0x02C (R/W) Interrupt Type Clear Register */
  __IOM  uint32_t  INTPOLSET;                /* Offset: 0x030 (R/W) Interrupt Polarity Set Register */
  __IOM  uint32_t  INTPOLCLR;                /* Offset: 0x034 (R/W) Interrupt Polarity Clear Register */
  union {
    __IM   uint32_t  INTSTATUS;              /* Offset: 0x038 (R/ ) Interrupt Status Register */
    __OM   uint32_t  INTCLEAR;               /* Offset: 0x038 ( /W) Interrupt Clear Register */
  };
         uint32_t RESERVED1[241];
  __IOM  uint32_t LB_MASKED[256];            /* Offset: 0x400 (R/W) Lower byte masked Access Register */
  __IOM  uint32_t UB_MASKED[256];            /* Offset: 0x800 (R/W) Upper byte masked Access Register */
} DS_CM3_GPIO_TypeDef;

/* DS_CM3_GPIO DATA Register Definitions */
#define DS_CM3_GPIO_DATA_Pos                    0                                                    /* DS_CM3_GPIO DATA: DATA position */
#define DS_CM3_GPIO_DATA_Msk                   (0xFFFFUL /*<< DS_CM3_GPIO_DATA_Pos*/)                /* DS_CM3_GPIO DATA: DATA mask */

/* DS_CM3_GPIO DATAOUT Register Definitions */
#define DS_CM3_GPIO_DATAOUT_Pos                 0                                                    /* DS_CM3_GPIO DATAOUT: DATAOUT position */
#define DS_CM3_GPIO_DATAOUT_Msk                (0xFFFFUL /*<< DS_CM3_GPIO_DATAOUT_Pos*/)             /* DS_CM3_GPIO DATAOUT: DATAOUT mask */

/* DS_CM3_GPIO OUTENSET Register Definitions */
#define DS_CM3_GPIO_OUTENSET_Pos                0                                                    /* DS_CM3_GPIO OUTEN: OUTEN position */
#define DS_CM3_GPIO_OUTENSET_Msk               (0xFFFFUL /*<< DS_CM3_GPIO_OUTEN_Pos*/)               /* DS_CM3_GPIO OUTEN: OUTEN mask */

/* DS_CM3_GPIO OUTENCLR Register Definitions */
#define DS_CM3_GPIO_OUTENCLR_Pos                0                                                    /* DS_CM3_GPIO OUTEN: OUTEN position */
#define DS_CM3_GPIO_OUTENCLR_Msk               (0xFFFFUL /*<< DS_CM3_GPIO_OUTEN_Pos*/)               /* DS_CM3_GPIO OUTEN: OUTEN mask */

/* DS_CM3_GPIO ALTFUNCSET Register Definitions */
#define DS_CM3_GPIO_ALTFUNCSET_Pos              0                                                    /* DS_CM3_GPIO ALTFUNC: ALTFUNC position */
#define DS_CM3_GPIO_ALTFUNCSET_Msk             (0xFFFFUL /*<< DS_CM3_GPIO_ALTFUNC_Pos*/)             /* DS_CM3_GPIO ALTFUNC: ALTFUNC mask */

/* DS_CM3_GPIO ALTFUNCCLR Register Definitions */
#define DS_CM3_GPIO_ALTFUNCCLR_Pos              0                                                    /* DS_CM3_GPIO ALTFUNC: ALTFUNC position */
#define DS_CM3_GPIO_ALTFUNCCLR_Msk             (0xFFFFUL /*<< DS_CM3_GPIO_ALTFUNC_Pos*/)             /* DS_CM3_GPIO ALTFUNC: ALTFUNC mask */

/* DS_CM3_GPIO INTENSET Register Definitions */
#define DS_CM3_GPIO_INTENSET_Pos                0                                                    /* DS_CM3_GPIO INTEN: INTEN position */
#define DS_CM3_GPIO_INTENSET_Msk               (0xFFFFUL /*<< DS_CM3_GPIO_INTEN_Pos*/)               /* DS_CM3_GPIO INTEN: INTEN mask */

/* DS_CM3_GPIO INTENCLR Register Definitions */
#define DS_CM3_GPIO_INTENCLR_Pos                0                                                    /* DS_CM3_GPIO INTEN: INTEN position */
#define DS_CM3_GPIO_INTENCLR_Msk               (0xFFFFUL /*<< DS_CM3_GPIO_INTEN_Pos*/)               /* DS_CM3_GPIO INTEN: INTEN mask */

/* DS_CM3_GPIO INTTYPESET Register Definitions */
#define DS_CM3_GPIO_INTTYPESET_Pos              0                                                    /* DS_CM3_GPIO INTTYPE: INTTYPE position */
#define DS_CM3_GPIO_INTTYPESET_Msk             (0xFFFFUL /*<< DS_CM3_GPIO_INTTYPE_Pos*/)             /* DS_CM3_GPIO INTTYPE: INTTYPE mask */

/* DS_CM3_GPIO INTTYPECLR Register Definitions */
#define DS_CM3_GPIO_INTTYPECLR_Pos              0                                                    /* DS_CM3_GPIO INTTYPE: INTTYPE position */
#define DS_CM3_GPIO_INTTYPECLR_Msk             (0xFFFFUL /*<< DS_CM3_GPIO_INTTYPE_Pos*/)             /* DS_CM3_GPIO INTTYPE: INTTYPE mask */

/* DS_CM3_GPIO INTPOLSET Register Definitions */
#define DS_CM3_GPIO_INTPOLSET_Pos               0                                                    /* DS_CM3_GPIO INTPOL: INTPOL position */
#define DS_CM3_GPIO_INTPOLSET_Msk              (0xFFFFUL /*<< DS_CM3_GPIO_INTPOL_Pos*/)              /* DS_CM3_GPIO INTPOL: INTPOL mask */

/* DS_CM3_GPIO INTPOLCLR Register Definitions */
#define DS_CM3_GPIO_INTPOLCLR_Pos               0                                                    /* DS_CM3_GPIO INTPOL: INTPOL position */
#define DS_CM3_GPIO_INTPOLCLR_Msk              (0xFFFFUL /*<< DS_CM3_GPIO_INTPOL_Pos*/)              /* DS_CM3_GPIO INTPOL: INTPOL mask */

/* DS_CM3_GPIO INTCLEAR Register Definitions */
#define DS_CM3_GPIO_INTSTATUS_Pos               0                                                    /* DS_CM3_GPIO INTSTATUS: INTSTATUS position */
#define DS_CM3_GPIO_INTCLEAR_Msk               (0xFFUL /*<< DS_CM3_GPIO_INTSTATUS_Pos*/)             /* DS_CM3_GPIO INTSTATUS: INTSTATUS mask */

/* DS_CM3_GPIO INTCLEAR Register Definitions */
#define DS_CM3_GPIO_INTCLEAR_Pos                0                                                    /* DS_CM3_GPIO INTCLEAR: INTCLEAR position */
#define DS_CM3_GPIO_INTCLEAR_Msk               (0xFFUL /*<< DS_CM3_GPIO_INTCLEAR_Pos*/)              /* DS_CM3_GPIO INTCLEAR: INTCLEAR mask */

/* DS_CM3_GPIO maskLOWBYTE Register Definitions */
#define DS_CM3_GPIO_MASKLOWBYTE_Pos             0                                                    /* DS_CM3_GPIO maskLOWBYTE: maskLOWBYTE position */
#define DS_CM3_GPIO_MASKLOWBYTE_Msk            (0x00FFUL /*<< DS_CM3_GPIO_MASKLOWBYTE_Pos*/)         /* DS_CM3_GPIO maskLOWBYTE: maskLOWBYTE mask */

/* DS_CM3_GPIO maskHIGHBYTE Register Definitions */
#define DS_CM3_GPIO_MASKHIGHBYTE_Pos            0                                                    /* DS_CM3_GPIO maskHIGHBYTE: maskHIGHBYTE position */
#define DS_CM3_GPIO_MASKHIGHBYTE_Msk           (0xFF00UL /*<< DS_CM3_GPIO_MASKHIGHBYTE_Pos*/)        /* DS_CM3_GPIO maskHIGHBYTE: maskHIGHBYTE mask */


/*------------- System Control --------------------------------------*/
typedef struct
{
  __IOM  uint32_t  REMAP;                    /* Offset: 0x000 (R/W) Remap Control Register */
  __IOM  uint32_t  PMUCTRL;                  /* Offset: 0x004 (R/W) PMU Control Register */
  __IOM  uint32_t  RESETOP;                  /* Offset: 0x008 (R/W) Reset Option Register */
         uint32_t  RESERVED0[1];
  __IOM  uint32_t  RSTINFO;                  /* Offset: 0x010 (R/W) Reset Information Register */
} DS_CM3_SYSCTRL_TypeDef;

/* DS_CM3_SYSCTRL REMAP Register Definitions */
#define DS_CM3_SYSCTRL_REMAP_Pos                 0                                                    /* DS_CM3_SYSCTRL REMAP: REMAP position */
#define DS_CM3_SYSCTRL_REMAP_Msk                (0x01UL /*<< DS_CM3_SYSCTRL_REMAP_Pos*/)              /* DS_CM3_SYSCTRL REMAP: REMAP mask */

/* DS_CM3_SYSCTRL PMUCTRL Register Definitions */
#define DS_CM3_SYSCTRL_PMUCTRL_EN_Pos            0                                                    /* DS_CM3_SYSCTRL PMUCTRL.EN: Enable psition */
#define DS_CM3_SYSCTRL_PMUCTRL_EN_Msk           (0x01UL /*<< DS_CM3_SYSCTRL_PMUCTRL_EN_Pos*/)         /* DS_CM3_SYSCTRL PMUCTRL.EN: Enable mask */

/* DS_CM3_SYSCTRL LOCKUPRST Register Definitions */
#define DS_CM3_SYSCTRL_LOCKUPRST_RESETOP_Pos     0                                                    /* DS_CM3_SYSCTRL LOCKUPRST.RESETOP: Reset enable position */
#define DS_CM3_SYSCTRL_LOCKUPRST_RESETOP_Msk    (0x01UL /*<< DS_CM3_SYSCTRL_LOCKUPRST_RESETOP_Pos*/)  /* DS_CM3_SYSCTRL LOCKUPRST.RESETOP: Reset enable mask */

/* DS_CM3_SYSCTRL RSTINFO Register Definitions */
#define DS_CM3_SYSCTRL_RSTINFO_LOCKUPRESET_Pos   2                                                    /* DS_CM3_SYSCTRL RSTINFO.LOCKUPRESET: LOCKUPRESET position */
#define DS_CM3_SYSCTRL_RSTINFO_LOCKUPRESET_Msk  (0x1UL << DS_CM3_SYSCTRL_RSTINFO_LOCKUPRESET_Pos)     /* DS_CM3_SYSCTRL RSTINFO.LOCKUPRESET: LOCKUPRESET mask */

#define DS_CM3_SYSCTRL_RSTINFO_WDOGRESETREQ_Pos  1                                                    /* DS_CM3_SYSCTRL RSTINFO.WDOGRESETREQ: Watchdog reset request position */
#define DS_CM3_SYSCTRL_RSTINFO_WDOGRESETREQ_Msk (0x1UL << DS_CM3_SYSCTRL_RSTINFO_WDOGRESETREQ_Pos)    /* DS_CM3_SYSCTRL RSTINFO.WDOGRESETREQ: Watchdog reset request mask */

#define DS_CM3_SYSCTRL_RSTINFO_SYSRESETREQ_Pos   0                                                    /* DS_CM3_SYSCTRL RSTINFO.SYSRESETREQ: Systemreset request position */
#define DS_CM3_SYSCTRL_RSTINFO_SYSRESETREQ_Msk  (0x1UL /*<< DS_CM3_SYSCTRL_RSTINFO_SYSRESETREQ_Pos*/) /* DS_CM3_SYSCTRL RSTINFO.SYSRESETREQ: Systemreset request mask */


/*------------------- FPGA system ----------------------------------------------*/
typedef struct /* see "ARM Cortex-M3 DesignStart FPGA User Guide r0p0" */
{
  __IOM  uint32_t LED;                       /* Offset: 0x000 (R/W)  LED connections */
         uint32_t RESERVED0[1];
  __IOM  uint32_t BUTTON;                    /* Offset: 0x008 (R/W)  Buttons */
         uint32_t RESERVED1[1];
  __IOM  uint32_t CLK1HZ;                    /* Offset: 0x010 (R/W)  1Hz up counter */
  __IOM  uint32_t CLK100HZ;                  /* Offset: 0x014 (R/W)  100Hz up counter */
  __IOM  uint32_t COUNTER;                   /* Offset: 0x018 (R/W)  Cycle Up Counter */
  __IOM  uint32_t PRESCALE;                  /* Offset: 0x01C (R/W)  Reload value for prescale counter */
  __IOM  uint32_t PSCNTR;                    /* Offset: 0x020 (R/W)  32-bit Prescale counter */
         uint32_t RESERVED3[10];
  __IOM  uint32_t MISC;                      /* Offset: 0x04C (R/W)  Miscellaneous control */
} DS_CM3_FPGASYS_TypeDef;

/* DS_CM3_FPGACTRL_LED Register Definitions */
#define DS_CM3_FPGASYS_LED_Pos                  0                                                    /* DS_CM3_FPGASYS LED: LED position */
#define DS_CM3_FPGASYS_LED_Msk                 (0x3UL /*<< DS_CM3_FPGACTRL_LED_Pos*/)                /* DS_CM3_FPGASYS LED: LED mask */

/* DS_CM3_FPGACTRL_BUTTON Register Definitions */
#define DS_CM3_FPGASYS_BUTTON_Pos               0                                                    /* DS_CM3_FPGASYS BUTTON: BUTTON position */
#define DS_CM3_FPGASYS_BUTTON_Msk              (0x3UL /*<< DS_CM3_FPGACTRL_BUTTON_Pos*/)             /* DS_CM3_FPGASYS BUTTON: BUTTON mask */

/* DS_CM3_FPGASYS_MISC Register Definitions */
#define DS_CM3_FPGASYS_MISC_CLCDBLCTRL_Pos      6                                                    /* DS_CM3_FPGASYS MISC.CLCDBLCTRL: CLCD_BL_CTRL position */
#define DS_CM3_FPGASYS_MISC_CLCDBLCTRL_Msk     (0x1UL << DS_CM3_FPGASYS_MISC_CLCDBLCTRL_Pos)         /* DS_CM3_FPGASYS MISC.CLCDBLCTRL: CLCD_BL_CTRL mask */

#define DS_CM3_FPGASYS_MISC_CLCDRD_Pos          5                                                    /* DS_CM3_FPGASYS MISC.CLCDRD: CLCD_RD position */
#define DS_CM3_FPGASYS_MISC_CLCDRD_Msk         (0x1UL << DS_CM3_FPGASYS_MISC_CLCDRD_Pos)             /* DS_CM3_FPGASYS MISC.CLCDRD: CLCD_RD mask */

#define DS_CM3_FPGASYS_MISC_CLCDRS_Pos          4                                                    /* DS_CM3_FPGASYS MISC.CLCDRS: CLCD_RS position */
#define DS_CM3_FPGASYS_MISC_CLCDRS_Msk         (0x1UL << DS_CM3_FPGASYS_MISC_CLCDRS_Pos)             /* DS_CM3_FPGASYS MISC.CLCDRS: CLCD_RS mask */

#define DS_CM3_FPGASYS_MISC_CLCDRESET_Pos       3                                                    /* DS_CM3_FPGASYS MISC.CLCDRESET: CLCD_RESET position */
#define DS_CM3_FPGASYS_MISC_CLCDRESET_Msk      (0x1UL << DS_CM3_FPGASYS_MISC_CLCDRESET_Pos)          /* DS_CM3_FPGASYS MISC.CLCDRESET: CLCD_RESET mask */

#define DS_CM3_FPGASYS_MISC_SPINSS_Pos          1                                                    /* DS_CM3_FPGASYS MISC.SPINSS: SPI_nSS position */
#define DS_CM3_FPGASYS_MISC_SPINSS_Msk         (0x1UL << DS_CM3_FPGASYS_MISC_SPINSS_Pos)             /* DS_CM3_FPGASYS MISC.SPINSS: SPI_nSS mask */

#define DS_CM3_FPGASYS_MISC_CLCDCS_Pos          0                                                    /* DS_CM3_FPGASYS MISC.CLCDCS: CLCD_CS position */
#define DS_CM3_FPGASYS_MISC_CLCDCS_Msk         (0x1UL /*<< DS_CM3_FPGASYS_MISC_CLCDCS_Pos*/)         /* DS_CM3_FPGASYS MISC.CLCDCS: CLCD_CS mask */


/*------------------- Audio I2S control ----------------------------------------------*/
typedef struct /* see "ARM Cortex-M3 DesignStart FPGA User Guide r0p0" */
{
  __IOM  uint32_t CONTROL;                   /* Offset: 0x000 (R/W) CONTROL Register */
  __IM   uint32_t STATUS;                    /* Offset: 0x004 (R/ ) STATUS Register */
  union {
    __IM   uint32_t ERROR;                   /* Offset: 0x008 (R/ ) Error Status Register */
    __OM   uint32_t ERRORCLR;                /* Offset: 0x008 ( /W) Error Clear Register */
  };
  __IOM  uint32_t DIVIDE;                    /* Offset: 0x00C (R/W) Divide ratio Register */
  __OM   uint32_t TXBUF;                     /* Offset: 0x010 ( /W) Transmit Buffer */
  __IM   uint32_t RXBUF;                     /* Offset: 0x014 (R/ ) Receive Buffer */
         uint32_t RESERVED1[186];
  __IOM  uint32_t ITCR;                      /* Offset: 0x300 (R/W) Integration Test Control Register */
  __OM   uint32_t ITIP1;                     /* Offset: 0x304 ( /W) Integration Test Input Register 1 */
  __OM   uint32_t ITOP1;                     /* Offset: 0x308 ( /W) Integration Test Output Register 1 */
} DS_CM3_I2S_TypeDef;

/* DS_CM3_I2S_CONTROL Register Definitions */
#define DS_CM3_I2S_CONTROL_CODECRST_Pos        17                                                    /* DS_CM3_I2S CONTROL.CODECRST: Codec reset position */
#define DS_CM3_I2S_CONTROL_CODECRST_Msk        (1UL << DS_CM3_I2S_CONTROL_CODECRST_Pos)              /* DS_CM3_I2S CONTROL.CODECRST: Codec reset mask */

#define DS_CM3_I2S_CONTROL_FIFORST_Pos         16                                                    /* DS_CM3_I2S CONTROL.FIFORST: FIFO reset position */
#define DS_CM3_I2S_CONTROL_FIFORST_Msk         (1UL << DS_CM3_I2S_CONTROL_FIFORST_Pos)               /* DS_CM3_I2S CONTROL.FIFORST: FIFO reset mask */

#define DS_CM3_I2S_CONTROL_RXWLVL_Pos          12                                                    /* DS_CM3_I2S CONTROL.RXWLVL: RX Buffer IRQ Water Level position */
#define DS_CM3_I2S_CONTROL_RXWLVL_Msk          (7UL << DS_CM3_I2S_CONTROL_RXWLVL_Pos)                /* DS_CM3_I2S CONTROL.RXWLVL: RX Buffer IRQ Water Level mask */

#define DS_CM3_I2S_CONTROL_TXWLVL_Pos           8                                                    /* DS_CM3_I2S CONTROL.TXWLVL: TX Buffer IRQ Water Level position */
#define DS_CM3_I2S_CONTROL_TXWLVL_Msk          (7UL << DS_CM3_I2S_CONTROL_TXWLVL_Pos)                /* DS_CM3_I2S CONTROL.TXWLVL: TX Buffer IRQ Water Level mask */

#define DS_CM3_I2S_CONTROL_RXINTEN_Pos          3                                                    /* DS_CM3_I2S CONTROL.RXINTEN: RX interrupt enable position */
#define DS_CM3_I2S_CONTROL_RXINTEN_Msk         (1UL << DS_CM3_I2S_CONTROL_RXINTEN_Pos)               /* DS_CM3_I2S CONTROL.RXINTEN: RX interrupt enable mask */

#define DS_CM3_I2S_CONTROL_RXEN_Pos             2                                                    /* DS_CM3_I2S CONTROL.RXEN: RX enable position */
#define DS_CM3_I2S_CONTROL_RXEN_Msk            (1UL << DS_CM3_I2S_CONTROL_RXEN_Pos)                  /* DS_CM3_I2S CONTROL.RXEN: RX enable mask */

#define DS_CM3_I2S_CONTROL_TXINTEN_Pos          1                                                    /* DS_CM3_I2S CONTROL.TXINTEN: TX interrupt enable position */
#define DS_CM3_I2S_CONTROL_TXINTEN_Msk         (1UL << DS_CM3_I2S_CONTROL_TXINTEN_Pos)               /* DS_CM3_I2S CONTROL.TXINTEN: TX interrupt enable mask */

#define DS_CM3_I2S_CONTROL_TXEN_Pos             0                                                    /* DS_CM3_I2S CONTROL.TXEN: TX enable position */
#define DS_CM3_I2S_CONTROL_TXEN_Msk            (1UL /*<< DS_CM3_I2S_CONTROL_TXEN_Pos*/)              /* DS_CM3_I2S CONTROL.TXEN: TX enable mask */

/* DS_CM3_I2S_STATUS Register Definitions */
#define DS_CM3_I2S_STATUS_RXFULL_Pos            5                                                    /* DS_CM3_I2S STATUS.RXFULL: RX Buffer Full position */
#define DS_CM3_I2S_STATUS_RXFULL_Msk           (1UL << DS_CM3_I2S_STATUS_RXFULL_Pos)                 /* DS_CM3_I2S STATUS.RXFULL: RX Buffer Full mask */

#define DS_CM3_I2S_STATUS_RXEMPTY_Pos           4                                                    /* DS_CM3_I2S STATUS.RXEMPTY: RX Buffer Empty position */
#define DS_CM3_I2S_STATUS_RXEMPTY_Msk          (1UL << DS_CM3_I2S_STATUS_RXEMPTY_Pos)                /* DS_CM3_I2S STATUS.RXEMPTY: RX Buffer Empty mask */

#define DS_CM3_I2S_STATUS_TXFULL_Pos            3                                                    /* DS_CM3_I2S STATUS.TXFULL: TX Buffer Full position */
#define DS_CM3_I2S_STATUS_TXFULL_Msk           (1UL << DS_CM3_I2S_STATUS_TXFULL_Pos)                 /* DS_CM3_I2S STATUS.TXFULL: TX Buffer Full mask */

#define DS_CM3_I2S_STATUS_TXEMPTY_Pos           2                                                    /* DS_CM3_I2S STATUS.TXEMPTY: TX Buffer Empty position */
#define DS_CM3_I2S_STATUS_TXEMPTY_Msk          (1UL << DS_CM3_I2S_STATUS_TXEMPTY_Pos)                /* DS_CM3_I2S STATUS.TXEMPTY: TX Buffer Empty mask */

#define DS_CM3_I2S_STATUS_RXINT_Pos             1                                                    /* DS_CM3_I2S STATUS.RXINT: RX Buffer Alert position */
#define DS_CM3_I2S_STATUS_RXINT_Msk            (1UL << DS_CM3_I2S_STATUS_RXINT_Pos)                  /* DS_CM3_I2S STATUS.RXINT: RX Buffer Alert mask */

#define DS_CM3_I2S_STATUS_TXINT_Pos             0                                                    /* DS_CM3_I2S STATUS.TXINT: TX Buffer Alert position */
#define DS_CM3_I2S_STATUS_TXINT_Msk            (1UL /*<< DS_CM3_I2S_STATUS_TXINT_Pos*/)              /* DS_CM3_I2S STATUS.TXINT: TX Buffer Alert mask */

/* DS_CM3_I2S_ERROR Register Definitions */
#define DS_CM3_I2S_ERROR_RXERR_Pos              1                                                    /* DS_CM3_I2S ERROR.RXERR: RX overrun position */
#define DS_CM3_I2S_ERROR_RXERR_Msk             (1UL << DS_CM3_I2S_ERROR_RXERR_Pos)                   /* DS_CM3_I2S ERROR.RXERR: RX overrun mask */

#define DS_CM3_I2S_ERROR_TXERR_Pos              0                                                    /* DS_CM3_I2S ERROR.TXERR: TX overrun or underrun position */
#define DS_CM3_I2S_ERROR_TXERR_Msk             (1UL /*<< DS_CM3_I2S_ERROR_TXERR_Pos*/)               /* DS_CM3_I2S ERROR.TXERR: TX overrun or underrun mask */


/*------------------- SCC ----------------------------------------------*/
typedef struct /* see "ARM Cortex-M3 DesignStart FPGA User Guide r0p0" */
{
  __IOM uint32_t CFG_REG0;                   /* Offset: 0x000 (R/W)  Remaps block RAM to ZBT */
  __IOM uint32_t CFG_REG1;                   /* Offset: 0x004 (R/W)  Controls the MCC user LEDs */
  __IOM uint32_t CFG_REG2;                   /* Offset: 0x004 (R/W)   */
  __IM  uint32_t CFG_REG3;                   /* Offset: 0x008 (R/ )  Denotes the state of the MCC user switches */
  __IM  uint32_t CFG_REG4;                   /* Offset: 0x00C (R/ )  Denotes the board revision */
        uint32_t RESERVED0[35];
  __IOM uint32_t SYS_CFGDATA_RTN;            /* Offset: 0x0A0 (R/W)  User data register */
  __IOM uint32_t SYS_CFGDATA_OUT;            /* Offset: 0x0A4 (R/W)  User data register */
  __IOM uint32_t SYS_CFGCTRL;                /* Offset: 0x0A8 (R/W)  Control register */
  __IOM uint32_t SYS_CFGSTAT;                /* Offset: 0x0AC (R/W)  Contains status information */
  __IOM uint32_t RESERVED1[20];
  __IOM uint32_t SCC_DLL;                    /* Offset: 0x100 (R/W)  DLL Lock Register */
        uint32_t RESERVED2[957];
  __IM  uint32_t SCC_AID;                    /* Offset: 0xFF8 (R/ )  SCC AID Register */
  __IM  uint32_t SCC_ID;                     /* Offset: 0xFFC (R/ )  Contains information about the FPGA image */
} DS_CM3_SCC_TypeDef;


/*------------------- TRNG ----------------------------------------------*/
typedef struct /* see "ARM Cortex-M3 DesignStart Eval RTL and Testbench User Guide r0p0" */
{
        uint32_t RESERVED0[64];
  __IOM uint32_t IMR;                        /* Offset: 0x100 (R/W)  Interrupt Masking Register */
  __IM  uint32_t ISR;                        /* Offset: 0x104 (R/ )  Interrupt Status Register */
  __OM  uint32_t ICR;                        /* Offset: 0x108 ( /W)  Interrupt status bit Clear Register */
  __IOM uint32_t CONFIG;                     /* Offset: 0x10C (R/W)  TRNG Configuration Register */
  __IM  uint32_t VALID;                      /* Offset: 0x110 (R/ )  TRNG Valid register */
  __IM  uint32_t EHR_DATA[6];                /* Offset: 0x114 (R/ )  EHR Data register */
  __IOM uint32_t RND_SOURCE_ENABLE;          /* Offset: 0x12C (R/W)  Random Source Enable register */
  __IOM uint32_t SAMPLE_CNT1;                /* Offset: 0x130 (R/W)  Sample Counter register */
  __IOM uint32_t AUTOCORR_STATISTIC;         /* Offset: 0x134 (R/W)  Autocorrelation Statistic register */
  __IM  uint32_t DEBUG_CONTROL;              /* Offset: 0x138 (R/ )  TRNG Debug Control register */
        uint32_t RESERVED1[1];
  __OM  uint32_t SW_RESET;                   /* Offset: 0x140 ( /W)  TRNG Software Reset Register */
        uint32_t RESERVED2[44];
  __IM  uint32_t BUSY;                       /* Offset: 0x1B8 (R/ )  TRNG Busy status register */
  __OM  uint32_t RST_BITS_COUNTER;           /* Offset: 0x1BC ( /W)  Reset Bits Counter register */
        uint32_t RESERVED3[8];
  __IM  uint32_t RST_BIST_COUNTER[3];        /* Offset: 0x1E0 (R/W)  RNG BIST Counter register*/
} DS_CM3_TRNG_TypeDef;


/*------------------- I2C ----------------------------------------------*/
typedef struct /* see Document ? */
{
  union {
    __OM  uint32_t CONTROLS;                 /* Offset: 0x000 ( /W)  CONTROL Set Register */
    __IM  uint32_t CONTROL;                  /* Offset: 0x000 (R/ )  CONTROL Status Register */
  };
  __OM  uint32_t CONTROLC;                   /* Offset: 0x004 ( /W)  CONTROL Clear Register */
} DS_CM3_I2C_TypeDef;

/* DS_CM3_I2C_CONTROLS Register Definitions */
#define DS_CM3_I2C_CONTROLS_SDA_Pos            1                                                     /* DS_CM3_I2C CONTROLS.SDA: SDA position */
#define DS_CM3_I2C_CONTROLS_SDA_Msk           (1UL << DS_CM3_I2C_CONTROLS_SDA_Pos)                   /* DS_CM3_I2C CONTROLS.SDA: SDA mask */

#define DS_CM3_I2C_CONTROLS_SCL_Pos             0                                                    /* DS_CM3_I2C CONTROLS.SCL: SCL position */
#define DS_CM3_I2C_CONTROLS_SCL_Msk            (1UL /*<< DS_CM3_I2C_CONTROLS_SCL_Pos*/)              /* DS_CM3_I2C CONTROLS.SCL: SCL mask */

/* DS_CM3_I2C_CONTROL Register Definitions */
#define DS_CM3_I2C_CONTROL_SDA_Pos              1                                                    /* DS_CM3_I2C CONTROL.SDA: SDA position */
#define DS_CM3_I2C_CONTROL_SDA_Msk             (1UL << DS_CM3_I2C_CONTROL_SDA_Pos)                   /* DS_CM3_I2C CONTROL.SDA: SDA mask */

#define DS_CM3_I2C_CONTROL_SCL_Pos              0                                                    /* DS_CM3_I2C CONTROL.SCL: SCL position */
#define DS_CM3_I2C_CONTROL_SCL_Msk             (1UL /*<< DS_CM3_I2C_CONTROL_SCL_Pos*/)               /* DS_CM3_I2C CONTROL.SCL: SCL mask */

/* DS_CM3_I2C_CONTROLC Register Definitions */
#define DS_CM3_I2C_CONTROLC_SDA_Pos             1                                                    /* DS_CM3_I2C CONTROLC.SDA: SDA position */
#define DS_CM3_I2C_CONTROLC_SDA_Msk            (1UL << DS_CM3_I2C_CONTROLC_SDA_Pos)                  /* DS_CM3_I2C CONTROLC.SDA: SDA mask */

#define DS_CM3_I2C_CONTROLC_SCL_Pos             0                                                    /* DS_CM3_I2C CONTROLC.SCL: SCL position */
#define DS_CM3_I2C_CONTROLC_SCL_Msk            (1UL /*<< DS_CM3_I2C_CONTROLC_SCL_Pos*/)              /* DS_CM3_I2C CONTROLC.SCL: SCL mask */


/*------------------- SSP (PL022) -----------------------------------------*/
typedef struct /* see "ARM PrimeCell Synchronous Serial Port (PL022) Technical Reference Manual r1p4" */
{
  __IOM uint32_t CR0;                        /* Offset: 0x000 (R/W)  Control register 0 */
  __IOM uint32_t CR1;                        /* Offset: 0x004 (R/W)  Control register 1 */
  __IOM uint32_t DR;                         /* Offset: 0x008 (R/W)  Data register */
  __IM  uint32_t SR;                         /* Offset: 0x00C (R/ )  Status register */
  __IOM uint32_t CPSR;                       /* Offset: 0x010 (R/W)  Clock prescale register */
  __IOM uint32_t IMSC;                       /* Offset: 0x014 (R/W)  Interrupt mask set or clear register */
  __IM  uint32_t RIS;                        /* Offset: 0x018 (R/ )  Raw interrupt status register */
  __IM  uint32_t MIS;                        /* Offset: 0x01C (R/ )  masked interrupt status register */
  __OM  uint32_t ICR;                        /* Offset: 0x020 ( /W)  Interrupt clear register */
  __IOM uint32_t DMACR;                      /* Offset: 0x024 (R/W)  DMA control register */
} DS_CM3_SSP_TypeDef;


/*------------------- RTC (PL031) -----------------------------------------*/
typedef struct /* see "ARM PrimeCell Real Time Clock (PL031) Technical Reference Manual" */
{
  __IM  uint32_t DR;                         /* Offset: 0x000 (R/ )  Data register */
  __IOM uint32_t MR;                         /* Offset: 0x004 (R/W)  Match register */
  __IOM uint32_t LR;                         /* Offset: 0x008 (R/W)  Load register */
  __IOM uint32_t CR;                         /* Offset: 0x00C (R/W)  Control register */
  __IOM uint32_t IMSC;                       /* Offset: 0x010 (R/W)  Interrupt mask set and clear register */
  __OM  uint32_t RIS;                        /* Offset: 0x014 (R/ )  Raw interrupt status register */
  __OM  uint32_t MIS;                        /* Offset: 0x018 (R/ )  Masked interrupt status register */
  __IOM uint32_t ICR;                        /* Offset: 0x01C ( /W)  Interrupt clear register */
} DS_CM3_RTC_TypeDef;


/* --------------------  End of section using anonymous unions  ------------------- */
#if   defined (__CC_ARM)
  #pragma pop
#elif defined (__ICCARM__)
  /* leave anonymous unions enabled */
#elif (__ARMCC_VERSION >= 6010050)
  #pragma clang diagnostic pop
#elif defined (__GNUC__)
  /* anonymous unions are enabled by defaULt */
#elif defined (__TMS470__)
  /* anonymous unions are enabled by defaULt */
#elif defined (__TASKING__)
  #pragma warning restore
#elif defined (__CSMC__)
  /* anonymous unions are enabled by defaULt */
#else
  #warning Not supported compiler type
#endif




/* ================================================================================ */
/* ================              Peripheral memory map             ================ */
/* ================================================================================ */

/* Peripheral and SRAM base address */
#define DS_CM3_FLASH_BASE        (0x00000000UL)
#define DS_CM3_SRAM_BASE         (0x20000000UL)
#define DS_CM3_PERIPH_BASE       (0x40000000UL)

#define DS_CM3_RAM_BASE          (0x20000000UL)
#define DS_CM3_APB_BASE          (0x40000000UL)
#define DS_CM3_AHB_BASE          (0x40010000UL)


/* APB peripherals */
#define DS_CM3_TIMER0_BASE       (DS_CM3_APB_BASE + 0x00000000UL)
#define DS_CM3_TIMER1_BASE       (DS_CM3_APB_BASE + 0x00001000UL)
#define DS_CM3_DUALTIMER_BASE    (DS_CM3_APB_BASE + 0x00002000UL)
#define DS_CM3_DUALTIMER1_BASE   (DS_CM3_DUALTIMER_BASE)
#define DS_CM3_DUALTIMER2_BASE   (DS_CM3_DUALTIMER_BASE + 0x20UL)
#define DS_CM3_UART0_BASE        (DS_CM3_APB_BASE + 0x00004000UL)
#define DS_CM3_UART1_BASE        (DS_CM3_APB_BASE + 0x00005000UL)
#define DS_CM3_RTC_BASE          (DS_CM3_APB_BASE + 0x00006000UL)
#define DS_CM3_WATCHDOG_BASE     (DS_CM3_APB_BASE + 0x00008000UL)
#define DS_CM3_TRNG_BASE         (DS_CM3_APB_BASE + 0x0000F000UL)


/* AHB peripherals */
#define DS_CM3_GPIO0_BASE        (DS_CM3_AHB_BASE + 0x00000000UL)
#define DS_CM3_GPIO1_BASE        (DS_CM3_AHB_BASE + 0x00001000UL)
#define DS_CM3_GPIO2_BASE        (DS_CM3_AHB_BASE + 0x00002000UL)
#define DS_CM3_GPIO3_BASE        (DS_CM3_AHB_BASE + 0x00003000UL)
#define DS_CM3_SYSCTRL_BASE      (DS_CM3_AHB_BASE + 0x0000F000UL)
#define DS_CM3_SPIEXT_BASE       (DS_CM3_AHB_BASE + 0x00010000UL)
#define DS_CM3_SPICLCD_BASE      (DS_CM3_AHB_BASE + 0x00011000UL)    /* SSP0 */
#define DS_CM3_I2CCLCD_BASE      (DS_CM3_AHB_BASE + 0x00012000UL)    /* I2C0 */
#define DS_CM3_I2CAUDIO_BASE     (DS_CM3_AHB_BASE + 0x00013000UL)
#define DS_CM3_I2S_BASE          (DS_CM3_AHB_BASE + 0x00014000UL)
#define DS_CM3_SPIADC_BASE       (DS_CM3_AHB_BASE + 0x00015000UL)
#define DS_CM3_SPISH0_BASE       (DS_CM3_AHB_BASE + 0x00016000UL)
#define DS_CM3_SPISH1_BASE       (DS_CM3_AHB_BASE + 0x00017000UL)
#define DS_CM3_FPGASYS_BASE      (DS_CM3_AHB_BASE + 0x00018000UL)
#define DS_CM3_I2CSH0_BASE       (DS_CM3_AHB_BASE + 0x00019000UL)
#define DS_CM3_I2CSH1_BASE       (DS_CM3_AHB_BASE + 0x0001A000UL)
#define DS_CM3_UART2_BASE        (DS_CM3_AHB_BASE + 0x0001C000UL)
#define DS_CM3_UART3_BASE        (DS_CM3_AHB_BASE + 0x0001D000UL)
#define DS_CM3_UART4_BASE        (DS_CM3_AHB_BASE + 0x0001E000UL)
#define DS_CM3_SCC_BASE          (DS_CM3_AHB_BASE + 0x0001F000UL)
#define DS_CM3_GPIO4_BASE        (DS_CM3_AHB_BASE + 0x00020000UL)
#define DS_CM3_GPIO5_BASE        (DS_CM3_AHB_BASE + 0x00021000UL)
#define DS_CM3_ETH_BASE          (                  0x40200000UL)
#define DS_CM3_VGACONSOLE_BASE   (                  0x41000000UL)
#define DS_CM3_VGAIMAGE_BASE     (                  0x41100000UL)


/* ================================================================================ */
/* ================             Peripheral declaration             ================ */
/* ================================================================================ */
#define DS_CM3_TIMER0       ((DS_CM3_TIMER_TypeDef            *) DS_CM3_TIMER0_BASE     )
#define DS_CM3_TIMER1       ((DS_CM3_TIMER_TypeDef            *) DS_CM3_TIMER1_BASE     )
#define DS_CM3_DUALTIMER    ((DS_CM3_DUALTIMER_TypeDef        *) DS_CM3_DUALTIMER_BASE  )
#define DS_CM3_DUALTIMER1   ((DS_CM3_DUALTIMER_SINGLE_TypeDef *) DS_CM3_DUALTIMER_1_BASE)
#define DS_CM3_DUALTIMER2   ((DS_CM3_DUALTIMER_SINGLE_TypeDef *) DS_CM3_DUALTIMER_2_BASE)
#define DS_CM3_UART0        ((DS_CM3_UART_TypeDef             *) DS_CM3_UART0_BASE      )
#define DS_CM3_UART1        ((DS_CM3_UART_TypeDef             *) DS_CM3_UART1_BASE      )
#define DS_CM3_RTC          ((DS_CM3_RTC_TypeDef              *) DS_CM3_RTC_BASE        )
#define DS_CM3_WATCHDOG     ((DS_CM3_WATCHDOG_TypeDef         *) DS_CM3_WATCHDOG_BASE   )
#define DS_CM3_TRNG         ((DS_CM3_TRNG_TypeDef             *) DS_CM3_TRNG_BASE       )
#define DS_CM3_GPIO0        ((DS_CM3_GPIO_TypeDef             *)  DS_CM3_GPIO0_BASE     )
#define DS_CM3_GPIO1        ((DS_CM3_GPIO_TypeDef             *)  DS_CM3_GPIO1_BASE     )
#define DS_CM3_GPIO2        ((DS_CM3_GPIO_TypeDef             *)  DS_CM3_GPIO2_BASE     )
#define DS_CM3_GPIO3        ((DS_CM3_GPIO_TypeDef             *)  DS_CM3_GPIO3_BASE     )
#define DS_CM3_SYSCTRL      ((DS_CM3_SYSCTRL_TypeDef          *)  DS_CM3_SYSCTRL_BASE   )
#define DS_CM3_SPIEXT       ((DS_CM3_SSP_TypeDef              *)  DS_CM3_SPIEXT_BASE    )
#define DS_CM3_SPICLCD      ((DS_CM3_SSP_TypeDef              *)  DS_CM3_SPICLCD_BASE   )
#define DS_CM3_I2CCLCD      ((DS_CM3_I2C_TypeDef              *)  DS_CM3_I2CCLCD_BASE   )
#define DS_CM3_I2CAUDIO     ((DS_CM3_I2C_TypeDef              *)  DS_CM3_I2CAUDIO_BASE  )
#define DS_CM3_I2S          ((DS_CM3_I2S_TypeDef              *)  DS_CM3_I2S_BASE       )
#define DS_CM3_SPIADC       ((DS_CM3_SSP_TypeDef              *)  DS_CM3_SPIADC_BASE    )
#define DS_CM3_SPISH0       ((DS_CM3_SSP_TypeDef              *)  DS_CM3_SPISH0_BASE    )
#define DS_CM3_SPISH1       ((DS_CM3_SSP_TypeDef              *)  DS_CM3_SPISH1_BASE    )
#define DS_CM3_FPGASYS      ((DS_CM3_FPGASYS_TypeDef          *)  DS_CM3_FPGASYS_BASE   )
#define DS_CM3_I2CSH0       ((DS_CM3_I2C_TypeDef              *)  DS_CM3_I2CSH0_BASE    )
#define DS_CM3_I2CSH1       ((DS_CM3_I2C_TypeDef              *)  DS_CM3_I2CSH1_BASE    )
#define DS_CM3_UART2        ((DS_CM3_UART_TypeDef             *)  DS_CM3_UART2_BASE     )
#define DS_CM3_UART3        ((DS_CM3_UART_TypeDef             *)  DS_CM3_UART3_BASE     )
#define DS_CM3_UART4        ((DS_CM3_UART_TypeDef             *)  DS_CM3_UART4_BASE     )
#define DS_CM3_SCC          ((DS_CM3_SCC_TypeDef              *)  DS_CM3_SCC_BASE       )
#define DS_CM3_GPIO4        ((DS_CM3_GPIO4_TypeDef            *)  DS_CM3_GPIO4_BASE     )
#define DS_CM3_GPIO5        ((DS_CM3_GPIO5_TypeDef            *)  DS_CM3_GPIO5_BASE     )


#ifdef __cplusplus
}
#endif

#endif  /* DS_CM3_H */
