/**************************************************************************//**
 * @file     startup_DS_CM3.S
 * @brief    CMSIS Core Device Startup File for
 *           DesignStart Cortex-M3 Device
 * @version  V1.0.0
 * @date     05. May 2017
 ******************************************************************************/
/* Copyright (c) 2011 - 2017 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

	.syntax	unified
	.arch	armv7-m

	.section .stack
	.align	3
#ifdef __STACK_SIZE
	.equ	Stack_Size, __STACK_SIZE
#else
	.equ	Stack_Size, 0x00000400
#endif
	.globl	__StackTop
	.globl	__StackLimit
__StackLimit:
	.space	Stack_Size
	.size	__StackLimit, . - __StackLimit
__StackTop:
	.size	__StackTop, . - __StackTop

	.section .heap
	.align	3
#ifdef __HEAP_SIZE
	.equ	Heap_Size, __HEAP_SIZE
#else
	.equ	Heap_Size, 0x00000C00
#endif
	.globl	__HeapBase
	.globl	__HeapLimit
__HeapBase:
	.if	Heap_Size
	.space	Heap_Size
	.endif
	.size	__HeapBase, . - __HeapBase
__HeapLimit:
	.size	__HeapLimit, . - __HeapLimit

	.section .vectors
	.align	2
	.globl	__Vectors
__Vectors:
	.long	__StackTop            /* Top of Stack */
	.long	Reset_Handler         /* Reset Handler */
	.long	NMI_Handler           /* NMI Handler */
	.long	HardFault_Handler     /* Hard Fault Handler */
	.long	MemManage_Handler     /* MPU Fault Handler */
	.long	BusFault_Handler      /* Bus Fault Handler */
	.long	UsageFault_Handler    /* Usage Fault Handler */
	.long	0                     /* Reserved */
	.long	0                     /* Reserved */
	.long	0                     /* Reserved */
	.long	0                     /* Reserved */
	.long	SVC_Handler           /* SVCall Handler */
	.long	DebugMon_Handler      /* Debug Monitor Handler */
	.long	0                     /* Reserved */
	.long	PendSV_Handler        /* PendSV Handler */
	.long	SysTick_Handler       /* SysTick Handler */

	/* External interrupts */
    .long   UART0_Handler         /*   0 UART0 TX/RX combined */
    .long   0                     /*   1 Unused */
    .long   UART1_Handler         /*   2 UART1 TX/RX combined */
    .long   RESERVED1_Handler     /*   3 Reserved for APB slaves */
    .long   RESERVED2_Handler     /*   4 Reserved for APB slaves */
    .long   RTC_Handler           /*   5 RTC */
    .long   GPIO0_Handler         /*   6 GPIO 0 combined */
    .long   GPIO1_Handler         /*   7 GPIO 1 combined */
    .long   TIMER0_Handler        /*   8 Timer0 */
    .long   TIMER1_Handler        /*   9 Timer1 */
    .long   DUALTIMER_Handler     /*  10 Dual Timer */
    .long   RESERVED3_Handler     /*  11 Reserved for APB slaves */
    .long   UART_OVF_Handler      /*  12 UART 0/1/2/3/4 overflow */
    .long   RESERVED4_Handler     /*  13 Reserved for APB slaves */
    .long   0                     /*  14 Unused */
    .long   TSC_Handler           /*  15 MPS2+ baseboard Touch Screen */
    .long   GPIO0_0_Handler       /*  16 GPIO0 individual interrupt ( 0) */
    .long   GPIO0_1_Handler       /*  17 GPIO0 individual interrupt ( 1) */
    .long   GPIO0_2_Handler       /*  18 GPIO0 individual interrupt ( 2) */
    .long   GPIO0_3_Handler       /*  19 GPIO0 individual interrupt ( 3) */
    .long   GPIO0_4_Handler       /*  20 GPIO0 individual interrupt ( 4) */
    .long   GPIO0_5_Handler       /*  21 GPIO0 individual interrupt ( 5) */
    .long   GPIO0_6_Handler       /*  22 GPIO0 individual interrupt ( 6) */
    .long   GPIO0_7_Handler       /*  23 GPIO0 individual interrupt ( 7) */
    .long   GPIO0_8_Handler       /*  24 GPIO0 individual interrupt ( 8) */
    .long   GPIO0_9_Handler       /*  25 GPIO0 individual interrupt ( 9) */
    .long   GPIO0_10_Handler      /*  26 GPIO0 individual interrupt (10) */
    .long   GPIO0_11_Handler      /*  27 GPIO0 individual interrupt (11) */
    .long   GPIO0_12_Handler      /*  28 GPIO0 individual interrupt (12) */
    .long   GPIO0_13_Handler      /*  29 GPIO0 individual interrupt (13) */
    .long   GPIO0_14_Handler      /*  30 GPIO0 individual interrupt (14) */
    .long   GPIO0_15_Handler      /*  31 GPIO0 individual interrupt (15) */
    .long   RESERVED5_Handler     /*  32 Reserved for Flash */
    .long   RESERVED6_Handler     /*  33 Reserved for Flash */
    .long   RESERVED7_Handler     /*  34 reserved for Cordio BT4 */
    .long   RESERVED8_Handler     /*  35 reserved for Cordio BT4 */
    .long   RESERVED9_Handler     /*  36 reserved for Cordio BT4 */
    .long   RESERVED10_Handler    /*  37 reserved for Cordio BT4 */
    .long   RESERVED11_Handler    /*  38 reserved for Cordio BT4 */
    .long   RESERVED12_Handler    /*  39 reserved for Cordio BT4 */
    .long   RESERVED13_Handler    /*  40 reserved for Cordio BT4 */
    .long   RESERVED14_Handler    /*  41 reserved for Cordio BT4 */
    .long   GPIO2_Handler         /*  42 Combined */
    .long   GPIO3_Handler         /*  43 Combined */
    .long   TRNG_Handler          /*  44 True Random Number Generator */
    .long   UART2_Handler         /*  45 UART 2 TX/RX compined */
    .long   UART3_Handler         /*  46 UART 3 TX/RX compined */
    .long   ETHERNET_Handler      /*  47 Ethernet Interrupt */
    .long   I2S_Handler           /*  48 I2S Interrupt */
    .long   MPS2_SPI0_Handler     /*  49 SPI Header interrupt */
    .long   MPS2_SPI1_Handler     /*  50 CLCD SPI interrupt */
    .long   MPS2_SPI2_Handler     /*  51 ADC for Shield */
    .long   MPS2_SPI3_Handler     /*  52 Shield0 SPI */
    .long   MPS2_SPI4_Handler     /*  53 Shield1 SPI */
    .long   GPIO4_Handler         /*  54 GPIO 4 combined */
    .long   GPIO5_Handler         /*  55 GPIO 5 combined */
    .long   UART4_Handler         /*  56 UART 4 TX/Rx combined */

	.size	__Vectors, . - __Vectors

	.text
	.thumb
	.thumb_func
	.align	2
	.globl	Reset_Handler
	.type	Reset_Handler, %function
Reset_Handler:
/*  Firstly it copies data from read only memory to RAM. There are two schemes
 *  to copy. One can copy more than one sections. Another can only copy
 *  one section.  The former scheme needs more instructions and read-only
 *  data to implement than the latter.
 *  Macro __STARTUP_COPY_MULTIPLE is used to choose between two schemes.  */

#ifdef __STARTUP_COPY_MULTIPLE
/*  Multiple sections scheme.
 *
 *  Between symbol address __copy_table_start__ and __copy_table_end__,
 *  there are array of triplets, each of which specify:
 *    offset 0: LMA of start of a section to copy from
 *    offset 4: VMA of start of a section to copy to
 *    offset 8: size of the section to copy. Must be multiply of 4
 *
 *  All addresses must be aligned to 4 bytes boundary.
 */
	ldr	r4, =__copy_table_start__
	ldr	r5, =__copy_table_end__

.L_loop0:
	cmp	r4, r5
	bge	.L_loop0_done
	ldr	r1, [r4]
	ldr	r2, [r4, #4]
	ldr	r3, [r4, #8]

.L_loop0_0:
	subs	r3, #4
	ittt	ge
	ldrge	r0, [r1, r3]
	strge	r0, [r2, r3]
	bge	.L_loop0_0

	adds	r4, #12
	b	.L_loop0

.L_loop0_done:
#else
/*  Single section scheme.
 *
 *  The ranges of copy from/to are specified by following symbols
 *    __etext: LMA of start of the section to copy from. Usually end of text
 *    __data_start__: VMA of start of the section to copy to
 *    __data_end__: VMA of end of the section to copy to
 *
 *  All addresses must be aligned to 4 bytes boundary.
 */
	ldr	r1, =__etext
	ldr	r2, =__data_start__
	ldr	r3, =__data_end__

.L_loop1:
	cmp	r2, r3
	ittt	lt
	ldrlt	r0, [r1], #4
	strlt	r0, [r2], #4
	blt	.L_loop1
#endif /*__STARTUP_COPY_MULTIPLE */

/*  This part of work usually is done in C library startup code. Otherwise,
 *  define this macro to enable it in this startup.
 *
 *  There are two schemes too. One can clear multiple BSS sections. Another
 *  can only clear one section. The former is more size expensive than the
 *  latter.
 *
 *  Define macro __STARTUP_CLEAR_BSS_MULTIPLE to choose the former.
 *  Otherwise efine macro __STARTUP_CLEAR_BSS to choose the later.
 */
#ifdef __STARTUP_CLEAR_BSS_MULTIPLE
/*  Multiple sections scheme.
 *
 *  Between symbol address __copy_table_start__ and __copy_table_end__,
 *  there are array of tuples specifying:
 *    offset 0: Start of a BSS section
 *    offset 4: Size of this BSS section. Must be multiply of 4
 */
	ldr	r3, =__zero_table_start__
	ldr	r4, =__zero_table_end__

.L_loop2:
	cmp	r3, r4
	bge	.L_loop2_done
	ldr	r1, [r3]
	ldr	r2, [r3, #4]
	movs	r0, 0

.L_loop2_0:
	subs	r2, #4
	itt	ge
	strge	r0, [r1, r2]
	bge	.L_loop2_0

	adds	r3, #8
	b	.L_loop2
.L_loop2_done:
#elif defined (__STARTUP_CLEAR_BSS)
/*  Single BSS section scheme.
 *
 *  The BSS section is specified by following symbols
 *    __bss_start__: start of the BSS section.
 *    __bss_end__: end of the BSS section.
 *
 *  Both addresses must be aligned to 4 bytes boundary.
 */
	ldr	r1, =__bss_start__
	ldr	r2, =__bss_end__

	movs	r0, 0
.L_loop3:
	cmp	r1, r2
	itt	lt
	strlt	r0, [r1], #4
	blt	.L_loop3
#endif /* __STARTUP_CLEAR_BSS_MULTIPLE || __STARTUP_CLEAR_BSS */

#ifndef __NO_SYSTEM_INIT
	bl	SystemInit
#endif

#ifndef __START
#define __START _start
#endif
	bl	__START

	.pool
	.size	Reset_Handler, . - Reset_Handler

	.align	1
	.thumb_func
	.weak	Default_Handler
	.type	Default_Handler, %function
Default_Handler:
	b	.
	.size	Default_Handler, . - Default_Handler

/*    Macro to define default handlers. Default handler
 *    will be weak symbol and just dead loops. They can be
 *    overwritten by other handlers */
	.macro	def_irq_handler	handler_name
	.weak	\handler_name
	.set	\handler_name, Default_Handler
	.endm

	def_irq_handler	NMI_Handler
	def_irq_handler	HardFault_Handler
	def_irq_handler	MemManage_Handler
	def_irq_handler	BusFault_Handler
	def_irq_handler	UsageFault_Handler
	def_irq_handler	SVC_Handler
	def_irq_handler	DebugMon_Handler
	def_irq_handler	PendSV_Handler
	def_irq_handler	SysTick_Handler

    def_irq_handler UART0_Handler
    def_irq_handler UART1_Handler
    def_irq_handler RESERVED1_Handler
    def_irq_handler RESERVED2_Handler
    def_irq_handler RTC_Handler
    def_irq_handler GPIO0_Handler
    def_irq_handler GPIO1_Handler
    def_irq_handler TIMER0_Handler
    def_irq_handler TIMER1_Handler
    def_irq_handler DUALTIMER_Handler
    def_irq_handler RESERVED3_Handler
    def_irq_handler UART_OVF_Handler
    def_irq_handler RESERVED4_Handler
    def_irq_handler TSC_Handler
    def_irq_handler GPIO0_0_Handler
    def_irq_handler GPIO0_1_Handler
    def_irq_handler GPIO0_2_Handler
    def_irq_handler GPIO0_3_Handler
    def_irq_handler GPIO0_4_Handler
    def_irq_handler GPIO0_5_Handler
    def_irq_handler GPIO0_6_Handler
    def_irq_handler GPIO0_7_Handler
    def_irq_handler GPIO0_8_Handler
    def_irq_handler GPIO0_9_Handler
    def_irq_handler GPIO0_10_Handler
    def_irq_handler GPIO0_11_Handler
    def_irq_handler GPIO0_12_Handler
    def_irq_handler GPIO0_13_Handler
    def_irq_handler GPIO0_14_Handler
    def_irq_handler GPIO0_15_Handler
    def_irq_handler RESERVED5_Handler
    def_irq_handler RESERVED6_Handler
    def_irq_handler RESERVED7_Handler
    def_irq_handler RESERVED8_Handler
    def_irq_handler RESERVED9_Handler
    def_irq_handler RESERVED10_Handler
    def_irq_handler RESERVED11_Handler
    def_irq_handler RESERVED12_Handler
    def_irq_handler RESERVED13_Handler
    def_irq_handler RESERVED14_Handler
    def_irq_handler GPIO2_Handler
    def_irq_handler GPIO3_Handler
    def_irq_handler TRNG_Handler
    def_irq_handler UART2_Handler
    def_irq_handler UART3_Handler
    def_irq_handler ETHERNET_Handler
    def_irq_handler I2S_Handler
    def_irq_handler MPS2_SPI0_Handler
    def_irq_handler MPS2_SPI1_Handler
    def_irq_handler MPS2_SPI2_Handler
    def_irq_handler MPS2_SPI3_Handler
    def_irq_handler MPS2_SPI4_Handler
    def_irq_handler GPIO4_Handler
    def_irq_handler GPIO5_Handler
    def_irq_handler UART4_Handler

	.end
