/**************************************************************************//**
 * @file     startup_DS_CM3.c
 * @brief    CMSIS Core Device Startup File for
 *           DesignStart Cortex-M3
 * @version  V1.0.0
 * @date     05. May 2017
 ******************************************************************************/
/* Copyright (c) 2011 - 2017 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

#include <stdint.h>


/*----------------------------------------------------------------------------
  Linker generated Symbols
 *----------------------------------------------------------------------------*/
extern uint32_t __etext;
extern uint32_t __data_start__;
extern uint32_t __data_end__;
extern uint32_t __copy_table_start__;
extern uint32_t __copy_table_end__;
extern uint32_t __zero_table_start__;
extern uint32_t __zero_table_end__;
extern uint32_t __bss_start__;
extern uint32_t __bss_end__;
extern uint32_t __StackTop;

/*----------------------------------------------------------------------------
  Exception / Interrupt Handler Function Prototype
 *----------------------------------------------------------------------------*/
typedef void( *pFunc )( void );


/*----------------------------------------------------------------------------
  External References
 *----------------------------------------------------------------------------*/
#ifndef __START
extern void  _start(void) __attribute__((noreturn));    /* PreeMain (C library entry point) */
#else
extern int  __START(void) __attribute__((noreturn));    /* main entry point */
#endif

#ifndef __NO_SYSTEM_INIT
extern void SystemInit (void);            /* CMSIS System Initialization      */
#endif


/*----------------------------------------------------------------------------
  Internal References
 *----------------------------------------------------------------------------*/
void Default_Handler(void);                          /* Default empty handler */
void Reset_Handler(void);                            /* Reset Handler */


/*----------------------------------------------------------------------------
  User Initial Stack & Heap
 *----------------------------------------------------------------------------*/
#ifndef __STACK_SIZE
  #define	__STACK_SIZE  0x00000400
#endif
static uint8_t stack[__STACK_SIZE] __attribute__ ((aligned(8), used, section(".stack")));

#ifndef __HEAP_SIZE
  #define	__HEAP_SIZE   0x00000C00
#endif
#if __HEAP_SIZE > 0
static uint8_t heap[__HEAP_SIZE]   __attribute__ ((aligned(8), used, section(".heap")));
#endif


/*----------------------------------------------------------------------------
  Exception / Interrupt Handler
 *----------------------------------------------------------------------------*/
/* Cortex-M3 Processor Exceptions */
void NMI_Handler         (void) __attribute__ ((weak, alias("Default_Handler")));
void HardFault_Handler   (void) __attribute__ ((weak, alias("Default_Handler")));
void MemManage_Handler   (void) __attribute__ ((weak, alias("Default_Handler")));
void BusFault_Handler    (void) __attribute__ ((weak, alias("Default_Handler")));
void UsageFault_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void SVC_Handler         (void) __attribute__ ((weak, alias("Default_Handler")));
void DebugMon_Handler    (void) __attribute__ ((weak, alias("Default_Handler")));
void PendSV_Handler      (void) __attribute__ ((weak, alias("Default_Handler")));
void SysTick_Handler     (void) __attribute__ ((weak, alias("Default_Handler")));

/* External interrupts */
void UART0_Handler      (void) __attribute__ ((weak, alias("Default_Handler")));
void UART1_Handler      (void) __attribute__ ((weak, alias("Default_Handler")));
void RESERVED1_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void RESERVED2_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void RTC_Handler        (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_Handler      (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO1_Handler      (void) __attribute__ ((weak, alias("Default_Handler")));
void TIMER0_Handler     (void) __attribute__ ((weak, alias("Default_Handler")));
void TIMER1_Handler     (void) __attribute__ ((weak, alias("Default_Handler")));
void DUALTIMER_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void RESERVED3_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void UART_OVF_Handler   (void) __attribute__ ((weak, alias("Default_Handler")));
void RESERVED4_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void TSC_Handler        (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_0_Handler    (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_1_Handler    (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_2_Handler    (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_3_Handler    (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_4_Handler    (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_5_Handler    (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_6_Handler    (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_7_Handler    (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_8_Handler    (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_9_Handler    (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_10_Handler   (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_11_Handler   (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_12_Handler   (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_13_Handler   (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_14_Handler   (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO0_15_Handler   (void) __attribute__ ((weak, alias("Default_Handler")));
void RESERVED5_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void RESERVED6_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void RESERVED7_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void RESERVED8_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void RESERVED9_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void RESERVED10_Handler (void) __attribute__ ((weak, alias("Default_Handler")));
void RESERVED11_Handler (void) __attribute__ ((weak, alias("Default_Handler")));
void RESERVED12_Handler (void) __attribute__ ((weak, alias("Default_Handler")));
void RESERVED13_Handler (void) __attribute__ ((weak, alias("Default_Handler")));
void RESERVED14_Handler (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO2_Handler      (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO3_Handler      (void) __attribute__ ((weak, alias("Default_Handler")));
void TRNG_Handler       (void) __attribute__ ((weak, alias("Default_Handler")));
void UART2_Handler      (void) __attribute__ ((weak, alias("Default_Handler")));
void UART3_Handler      (void) __attribute__ ((weak, alias("Default_Handler")));
void ETHERNET_Handler   (void) __attribute__ ((weak, alias("Default_Handler")));
void I2S_Handler        (void) __attribute__ ((weak, alias("Default_Handler")));
void MPS2_SPI0_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void MPS2_SPI1_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void MPS2_SPI2_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void MPS2_SPI3_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void MPS2_SPI4_Handler  (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO4_Handler      (void) __attribute__ ((weak, alias("Default_Handler")));
void GPIO5_Handler      (void) __attribute__ ((weak, alias("Default_Handler")));
void UART4_Handler      (void) __attribute__ ((weak, alias("Default_Handler")));


/*----------------------------------------------------------------------------
  Exception / Interrupt Vector table
 *----------------------------------------------------------------------------*/
const pFunc __Vectors[] __attribute__ ((section(".vectors"))) = {
  /* Cortex-M3 Exceptions Handler */
  (pFunc)((uint32_t)&__StackTop),           /*     Initial Stack Pointer */
  Reset_Handler,                            /*     Reset Handler */
  NMI_Handler,                              /*     NMI Handler */
  HardFault_Handler,                        /*     Hard Fault Handler */
  MemManage_Handler,                        /*     MPU Fault Handler */
  BusFault_Handler,                         /*     Bus Fault Handler */
  UsageFault_Handler,                       /*     Usage Fault Handler */
  0,                                        /*     Reserved */
  0,                                        /*     Reserved */
  0,                                        /*     Reserved */
  0,                                        /*     Reserved */
  SVC_Handler,                              /*     SVCall Handler */
  DebugMon_Handler,                         /*     Debug Monitor Handler */
  0,                                        /*     Reserved */
  PendSV_Handler,                           /*     PendSV Handler */
  SysTick_Handler,                          /*     SysTick Handler */

  /* External interrupts */
  UART0_Handler,                            /*   0 UART0 TX/RX combined */
  0,                                        /*   1 Unused */
  UART1_Handler,                            /*   2 UART1 TX/RX combined */
  RESERVED1_Handler,                        /*   3 Reserved for APB slaves */
  RESERVED2_Handler,                        /*   4 Reserved for APB slaves */
  RTC_Handler,                              /*   5 RTC */
  GPIO0_Handler,                            /*   6 GPIO 0 combined */
  GPIO1_Handler,                            /*   7 GPIO 1 combined */
  TIMER0_Handler,                           /*   8 Timer0 */
  TIMER1_Handler,                           /*   9 Timer1 */
  DUALTIMER_Handler,                        /*  10 Dual Timer */
  RESERVED3_Handler,                        /*  11 Reserved for APB slaves */
  UART_OVF_Handler,                         /*  12 UART 0/1/2/3/4 overflow */
  RESERVED4_Handler,                        /*  13 Reserved for APB slaves */
  0,                                        /*  14 Unused */
  TSC_Handler,                              /*  15 MPS2+ baseboard Touch Screen */
  GPIO0_0_Handler,                          /*  16 GPIO0 individual interrupt ( 0) */
  GPIO0_1_Handler,                          /*  17 GPIO0 individual interrupt ( 1) */
  GPIO0_2_Handler,                          /*  18 GPIO0 individual interrupt ( 2) */
  GPIO0_3_Handler,                          /*  19 GPIO0 individual interrupt ( 3) */
  GPIO0_4_Handler,                          /*  20 GPIO0 individual interrupt ( 4) */
  GPIO0_5_Handler,                          /*  21 GPIO0 individual interrupt ( 5) */
  GPIO0_6_Handler,                          /*  22 GPIO0 individual interrupt ( 6) */
  GPIO0_7_Handler,                          /*  23 GPIO0 individual interrupt ( 7) */
  GPIO0_8_Handler,                          /*  24 GPIO0 individual interrupt ( 8) */
  GPIO0_9_Handler,                          /*  25 GPIO0 individual interrupt ( 9) */
  GPIO0_10_Handler,                         /*  26 GPIO0 individual interrupt (10) */
  GPIO0_11_Handler,                         /*  27 GPIO0 individual interrupt (11) */
  GPIO0_12_Handler,                         /*  28 GPIO0 individual interrupt (12) */
  GPIO0_13_Handler,                         /*  29 GPIO0 individual interrupt (13) */
  GPIO0_14_Handler,                         /*  30 GPIO0 individual interrupt (14) */
  GPIO0_15_Handler,                         /*  31 GPIO0 individual interrupt (15) */
  RESERVED5_Handler,                        /*  32 Reserved for Flash */
  RESERVED6_Handler,                        /*  33 Reserved for Flash */
  RESERVED7_Handler,                        /*  34 reserved for Cordio BT4 */
  RESERVED8_Handler,                        /*  35 reserved for Cordio BT4 */
  RESERVED9_Handler,                        /*  36 reserved for Cordio BT4 */
  RESERVED10_Handler,                       /*  37 reserved for Cordio BT4 */
  RESERVED11_Handler,                       /*  38 reserved for Cordio BT4 */
  RESERVED12_Handler,                       /*  39 reserved for Cordio BT4 */
  RESERVED13_Handler,                       /*  40 reserved for Cordio BT4 */
  RESERVED14_Handler,                       /*  41 reserved for Cordio BT4 */
  GPIO2_Handler,                            /*  42 Combined */
  GPIO3_Handler,                            /*  43 Combined */
  TRNG_Handler,                             /*  44 True Random Number Generator */
  UART2_Handler,                            /*  45 UART 2 TX/RX compined */
  UART3_Handler,                            /*  46 UART 3 TX/RX compined */
  ETHERNET_Handler,                         /*  47 Ethernet Interrupt */
  I2S_Handler,                              /*  48 I2S Interrupt */
  MPS2_SPI0_Handler,                        /*  49 SPI Header interrupt */
  MPS2_SPI1_Handler,                        /*  50 CLCD SPI interrupt */
  MPS2_SPI2_Handler,                        /*  51 ADC for Shield */
  MPS2_SPI3_Handler,                        /*  52 Shield0 SPI */
  MPS2_SPI4_Handler,                        /*  53 Shield1 SPI */
  GPIO4_Handler,                            /*  54 GPIO 4 combined */
  GPIO5_Handler,                            /*  55 GPIO 5 combined */
  UART4_Handler,                            /*  56 UART 4 TX/Rx combined */
};


/*----------------------------------------------------------------------------
  Reset Handler called on controller reset
 *----------------------------------------------------------------------------*/
void Reset_Handler(void) {
  uint32_t *pSrc, *pDest;
  uint32_t *pTable __attribute__((unused));

/*  Firstly it copies data from read only memory to RAM. There are two schemes
 *  to copy. One can copy more than one sections. Another can only copy
 *  one section.  The former scheme needs more instructions and read-only
 *  data to implement than the latter.
 *  Macro __STARTUP_COPY_MULTIPLE is used to choose between two schemes.  */

#ifdef __STARTUP_COPY_MULTIPLE
/*  Multiple sections scheme.
 *
 *  Between symbol address __copy_table_start__ and __copy_table_end__,
 *  there are array of triplets, each of which specify:
 *    offset 0: LMA of start of a section to copy from
 *    offset 4: VMA of start of a section to copy to
 *    offset 8: size of the section to copy. Must be multiply of 4
 *
 *  All addresses must be aligned to 4 bytes boundary.
 */
  pTable = &__copy_table_start__;

  for (; pTable < &__copy_table_end__; pTable = pTable + 3) {
		pSrc  = (uint32_t*)*(pTable + 0);
		pDest = (uint32_t*)*(pTable + 1);
		for (; pDest < (uint32_t*)(*(pTable + 1) + *(pTable + 2)) ; ) {
      *pDest++ = *pSrc++;
		}
	}
#else
/*  Single section scheme.
 *
 *  The ranges of copy from/to are specified by following symbols
 *    __etext: LMA of start of the section to copy from. Usually end of text
 *    __data_start__: VMA of start of the section to copy to
 *    __data_end__: VMA of end of the section to copy to
 *
 *  All addresses must be aligned to 4 bytes boundary.
 */
  pSrc  = &__etext;
  pDest = &__data_start__;

  for ( ; pDest < &__data_end__ ; ) {
    *pDest++ = *pSrc++;
  }
#endif /*__STARTUP_COPY_MULTIPLE */

/*  This part of work usually is done in C library startup code. Otherwise,
 *  define this macro to enable it in this startup.
 *
 *  There are two schemes too. One can clear multiple BSS sections. Another
 *  can only clear one section. The former is more size expensive than the
 *  latter.
 *
 *  Define macro __STARTUP_CLEAR_BSS_MULTIPLE to choose the former.
 *  Otherwise efine macro __STARTUP_CLEAR_BSS to choose the later.
 */
#ifdef __STARTUP_CLEAR_BSS_MULTIPLE
/*  Multiple sections scheme.
 *
 *  Between symbol address __copy_table_start__ and __copy_table_end__,
 *  there are array of tuples specifying:
 *    offset 0: Start of a BSS section
 *    offset 4: Size of this BSS section. Must be multiply of 4
 */
  pTable = &__zero_table_start__;

  for (; pTable < &__zero_table_end__; pTable = pTable + 2) {
		pDest = (uint32_t*)*(pTable + 0);
		for (; pDest < (uint32_t*)(*(pTable + 0) + *(pTable + 1)) ; ) {
      *pDest++ = 0;
		}
	}
#elif defined (__STARTUP_CLEAR_BSS)
/*  Single BSS section scheme.
 *
 *  The BSS section is specified by following symbols
 *    __bss_start__: start of the BSS section.
 *    __bss_end__: end of the BSS section.
 *
 *  Both addresses must be aligned to 4 bytes boundary.
 */
  pDest = &__bss_start__;

  for ( ; pDest < &__bss_end__ ; ) {
    *pDest++ = 0ul;
  }
#endif /* __STARTUP_CLEAR_BSS_MULTIPLE || __STARTUP_CLEAR_BSS */

#ifndef __NO_SYSTEM_INIT
	SystemInit();
#endif

#ifndef __START
#define __START _start
#endif
	__START();

}


/*----------------------------------------------------------------------------
  Default Handler for Exceptions / Interrupts
 *----------------------------------------------------------------------------*/
void Default_Handler(void) {

	while(1);
}
